@@ -96,21 +96,27 @@ static struct {
   { "logo-image",             CONFIGSET_SKIN },
   { "background-mimetype",    CONFIGSET_SKIN },
   { "background-image",       CONFIGSET_SKIN },
+  { "icon-mimetype",          CONFIGSET_SKIN },
+  { "icon-image",             CONFIGSET_SKIN },
   { "timeline-block-markup",  CONFIGSET_SKIN },
   { "timeline-date-format",   CONFIGSET_SKIN },
+  { "timeline-default-style", CONFIGSET_SKIN },
   { "timeline-dwelltime",     CONFIGSET_SKIN },
   { "timeline-closetime",     CONFIGSET_SKIN },
   { "timeline-max-comment",   CONFIGSET_SKIN },
   { "timeline-plaintext",     CONFIGSET_SKIN },
   { "timeline-truncate-at-blank", CONFIGSET_SKIN },
+  { "timeline-tslink-info",   CONFIGSET_SKIN },
   { "timeline-utc",           CONFIGSET_SKIN },
   { "adunit",                 CONFIGSET_SKIN },
   { "adunit-omit-if-admin",   CONFIGSET_SKIN },
   { "adunit-omit-if-user",    CONFIGSET_SKIN },
+  { "default-csp",            CONFIGSET_SKIN },
   { "sitemap-docidx",         CONFIGSET_SKIN },
   { "sitemap-download",       CONFIGSET_SKIN },
   { "sitemap-license",        CONFIGSET_SKIN },
   { "sitemap-contact",        CONFIGSET_SKIN },
+  { "safe-html",              CONFIGSET_SKIN },
 
 #ifdef FOSSIL_ENABLE_TH1_DOCS
   { "th1-docs",               CONFIGSET_TH1 },
@@ -139,17 +145,17 @@ static struct {
   { "crnl-glob",              CONFIGSET_PROJ },
   { "encoding-glob",          CONFIGSET_PROJ },
   { "empty-dirs",             CONFIGSET_PROJ },
+#ifdef FOSSIL_LEGACY_ALLOW_SYMLINKS
   { "allow-symlinks",         CONFIGSET_PROJ },
+#endif
   { "dotfiles",               CONFIGSET_PROJ },
   { "parent-project-code",    CONFIGSET_PROJ },
   { "parent-project-name",    CONFIGSET_PROJ },
   { "hash-policy",            CONFIGSET_PROJ },
   { "comment-format",         CONFIGSET_PROJ },
-
-#ifdef FOSSIL_ENABLE_LEGACY_MV_RM
+  { "mimetypes",              CONFIGSET_PROJ },
+  { "forbid-delta-manifests", CONFIGSET_PROJ },
   { "mv-rm-files",            CONFIGSET_PROJ },
-#endif
-
   { "ticket-table",           CONFIGSET_TKT  },
   { "ticket-common",          CONFIGSET_TKT  },
   { "ticket-change",          CONFIGSET_TKT  },
@@ -701,43 +707,43 @@ static void export_config(
 ** Where METHOD is one of: export import merge pull push reset.  All methods
 ** accept the -R or --repository option to specify a repository.
 **
-**    %fossil configuration export AREA FILENAME
+** >  fossil configuration export AREA FILENAME
 **
 **         Write to FILENAME exported configuration information for AREA.
 **         AREA can be one of:
 **
 **             all email project shun skin ticket user alias subscriber
 **
-**    %fossil configuration import FILENAME
+** >  fossil configuration import FILENAME
 **
 **         Read a configuration from FILENAME, overwriting the current
 **         configuration.
 **
-**    %fossil configuration merge FILENAME
+** >  fossil configuration merge FILENAME
 **
 **         Read a configuration from FILENAME and merge its values into
 **         the current configuration.  Existing values take priority over
 **         values read from FILENAME.
 **
-**    %fossil configuration pull AREA ?URL?
+** >  fossil configuration pull AREA ?URL?
 **
 **         Pull and install the configuration from a different server
 **         identified by URL.  If no URL is specified, then the default
 **         server is used.  Use the --overwrite flag to completely
 **         replace local settings with content received from URL.
 **
-**    %fossil configuration push AREA ?URL?
+** >  fossil configuration push AREA ?URL?
 **
 **         Push the local configuration into the remote server identified
 **         by URL.  Admin privilege is required on the remote server for
 **         this to work.  When the same record exists both locally and on
 **         the remote end, the one that was most recently changed wins.
 **
-**    %fossil configuration reset AREA
+** >  fossil configuration reset AREA
 **
 **         Restore the configuration to the default.  AREA as above.
 **
-**    %fossil configuration sync AREA ?URL?
+** >  fossil configuration sync AREA ?URL?
 **
 **         Synchronize configuration changes in the local repository with
 **         the remote repository at URL.
@@ -745,7 +751,7 @@ static void export_config(
 ** Options:
 **    -R|--repository FILE       Extract info from repository FILE
 **
-** See also: settings, unset
+** See also: [[settings]], [[unset]]
 */
 void configuration_cmd(void){
   int n;
@@ -815,11 +821,11 @@ void configuration_cmd(void){
     url_enable_proxy("via proxy: ");
     if( overwriteFlag ) mask |= CONFIGSET_OVERWRITE;
     if( strncmp(zMethod, "push", n)==0 ){
-      client_sync(0,0,(unsigned)mask);
+      client_sync(0,0,(unsigned)mask,0);
     }else if( strncmp(zMethod, "pull", n)==0 ){
-      client_sync(0,(unsigned)mask,0);
+      client_sync(0,(unsigned)mask,0,0);
     }else{
-      client_sync(0,(unsigned)mask,(unsigned)mask);
+      client_sync(0,(unsigned)mask,(unsigned)mask,0);
     }
   }else
   if( strncmp(zMethod, "reset", n)==0 ){
