@@ -4,7 +4,7 @@
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the Simplified BSD License (also
 ** known as the "2-Clause License" or "FreeBSD License".)
-
+**
 ** This program is distributed in the hope that it will be useful,
 ** but without any warranty; without even the implied warranty of
 ** merchantability or fitness for a particular purpose.
@@ -41,7 +41,7 @@ const char *mimetype_from_content(Blob *pBlob){
   */
   static const struct {
     const char *zPrefix;       /* The file prefix */
-    int size;                  /* Length of the prefix */
+    const int size;            /* Length of the prefix */
     const char *zMimetype;     /* The corresponding mimetype */
   } aMime[] = {
     { "GIF87a",                  6, "image/gif"  },
@@ -149,7 +149,7 @@ static const struct {
   { "jpe",        3, "image/jpeg"                        },
   { "jpeg",       4, "image/jpeg"                        },
   { "jpg",        3, "image/jpeg"                        },
-  { "js",         2, "application/x-javascript"          },
+  { "js",         2, "application/javascript"            },
   { "kar",        3, "audio/midi"                        },
   { "latex",      5, "application/x-latex"               },
   { "lha",        3, "application/octet-stream"          },
@@ -271,6 +271,7 @@ static const struct {
   { "vrml",       4, "model/vrml"                        },
   { "wav",        3, "audio/x-wav"                       },
   { "wax",        3, "audio/x-ms-wax"                    },
+  { "webp",       4, "image/webp"                        },
   { "wiki",       4, "text/x-fossil-wiki"                },
   { "wma",        3, "audio/x-ms-wma"                    },
   { "wmv",        3, "video/x-ms-wmv"                    },
@@ -306,6 +307,99 @@ static void mimetype_verify(void){
   }
 }
 
+/*
+** Looks in the contents of the "mimetypes" setting for a suffix
+** matching zSuffix. If found, it returns the configured value
+** in memory owned by the app (i.e. do not free() it), else it
+** returns 0.
+**
+** The mimetypes setting is expected to be a list of file extensions
+** and mimetypes, with one such mapping per line. A leading '.'  on
+** extensions is permitted for compatibility with lists imported from
+** other tools which require them.
+*/
+static const char *mimetype_from_name_custom(const char *zSuffix){
+  static char * zList = 0;
+  static char const * zEnd = 0;
+  static int once = 0;
+  char * z;
+  int tokenizerState /* 0=expecting a key, 1=skip next token,
+                     ** 2=accept next token */;
+  if(once==0){
+    once = 1; 
+    zList = db_get("mimetypes",0);
+    if(zList==0){
+      return 0;
+    }
+    /* Transform zList to simplify the main loop:
+       replace non-newline spaces with NUL bytes. */
+    zEnd = zList + strlen(zList);
+    for(z = zList; z<zEnd; ++z){
+      if('\n'==*z) continue;
+      else if(fossil_isspace(*z)){
+        *z = 0;
+      }
+    }
+  }else if(zList==0){
+    return 0;
+  }
+  tokenizerState = 0;
+  z = zList;
+  while( z<zEnd ){
+    if(*z==0){
+      ++z;
+      continue;
+    }
+    else if('\n'==*z){
+      if(2==tokenizerState){
+        /* We were expecting a value for a successful match
+           here, but got no value. Bail out. */
+        break;
+      }else{
+        /* May happen on malformed inputs. Skip this record. */
+        tokenizerState = 0;
+        ++z;
+        continue;
+      }
+    }
+    switch(tokenizerState){
+      case 0:{ /* This is a file extension */
+        static char * zCase = 0;
+        if('.'==*z){
+          /*ignore an optional leading dot, for compatibility
+            with some external mimetype lists*/;
+          if(++z==zEnd){
+            break;
+          }
+        }
+        if(zCase<z){
+          /*we have not yet case-folded this section: lower-case it*/
+          for(zCase = z; zCase<zEnd && *zCase!=0; ++zCase){
+            if(!(0x80 & *zCase)){
+              *zCase = (char)fossil_tolower(*zCase);
+            }
+          }
+        }
+        if(strcmp(z,zSuffix)==0){
+          tokenizerState = 2 /* Match: accept the next value. */;
+        }else{
+          tokenizerState = 1 /* No match: skip the next value. */;
+        }
+        z += strlen(z);
+        break;
+      }
+      case 1: /* This is a value, but not a match. Skip it. */
+        z += strlen(z);
+        break;
+      case 2: /* This is the value which matched the previous key. */;
+        return z;
+      default:
+        assert(!"cannot happen - invalid tokenizerState value.");
+    }
+  }
+  return 0;
+}
+
 /*
 ** Guess the mime-type of a document based on its name.
 */
@@ -335,6 +429,10 @@ const char *mimetype_from_name(const char *zName){
   if( len<sizeof(zSuffix)-1 ){
     sqlite3_snprintf(sizeof(zSuffix), zSuffix, "%s", z);
     for(i=0; zSuffix[i]; i++) zSuffix[i] = fossil_tolower(zSuffix[i]);
+    z = mimetype_from_name_custom(zSuffix);
+    if(z!=0){
+      return z;
+    }
     first = 0;
     last = count(aMime) - 1;
     while( first<=last ){
@@ -366,6 +464,7 @@ const char *mimetype_from_name(const char *zName){
 void mimetype_test_cmd(void){
   int i;
   mimetype_verify();
+  db_find_and_open_repository(0, 0);
   for(i=2; i<g.argc; i++){
     fossil_print("%-20s -> %s\n", g.argv[i], mimetype_from_name(g.argv[i]));
   }
@@ -379,11 +478,50 @@ void mimetype_test_cmd(void){
 */
 void mimetype_list_page(void){
   int i;
+  char *zCustomList = 0;    /* value of the mimetypes setting */
+  int nCustomEntries = 0;   /* number of entries in the mimetypes
+                            ** setting */
   mimetype_verify();
   style_header("Mimetype List");
   @ <p>The Fossil <a href="%R/help?cmd=/doc">/doc</a> page uses filename
-  @ suffixes and the following table to guess at the appropriate mimetype
-  @ for each document.</p>
+  @ suffixes and the following tables to guess at the appropriate mimetype
+  @ for each document. Mimetypes may be customized and overridden using
+  @ <a href="%R/help?cmd=mimetypes">the mimetypes config setting</a>.</p>
+  zCustomList = db_get("mimetypes",0);
+  if( zCustomList!=0 ){
+    Blob list, entry, key, val;
+    @ <h1>Repository-specific mimetypes</h1>
+    @ <p>The following extension-to-mimetype mappings are defined via
+    @ the <a href="%R/help?cmd=mimetypes">mimetypes setting</a>.</p>
+    @ <table class='sortable mimetypetable' border=1 cellpadding=0 \
+    @ data-column-types='tt' data-init-sort='0'>
+    @ <thead>
+    @ <tr><th>Suffix<th>Mimetype
+    @ </thead>
+    @ <tbody>
+    blob_set(&list, zCustomList);
+    while( blob_line(&list, &entry)>0 ){
+      const char *zKey;
+      if( blob_token(&entry, &key)==0 ) continue;
+      if( blob_token(&entry, &val)==0 ) continue;
+      zKey = blob_str(&key);
+      if( zKey[0]=='.' ) zKey++;
+      @ <tr><td>%h(zKey)<td>%h(blob_str(&val))</tr>
+      nCustomEntries++;
+    }
+    fossil_free(zCustomList);
+    if( nCustomEntries==0 ){
+      /* This can happen if the option is set to an empty/space-only
+      ** value. */
+      @ <tr><td colspan="2"><em>none</em></tr>
+    }
+    @ </tbody></table>
+  }
+  @ <h1>Default built-in mimetypes</h1>
+  if(nCustomEntries>0){
+    @ <p>Entries starting with an exclamation mark <em><strong>!</strong></em>
+    @ are overwritten by repository-specific settings.</p>
+  }
   @ <table class='sortable mimetypetable' border=1 cellpadding=0 \
   @ data-column-types='tt' data-init-sort='1'>
   @ <thead>
@@ -391,7 +529,12 @@ void mimetype_list_page(void){
   @ </thead>
   @ <tbody>
   for(i=0; i<count(aMime); i++){
-    @ <tr><td>%h(aMime[i].zSuffix)<td>%h(aMime[i].zMimetype)</tr>
+    const char *zFlag = "";
+    if(nCustomEntries>0 &&
+       mimetype_from_name_custom(aMime[i].zSuffix)!=0){
+      zFlag = "<em><strong>!</strong></em> ";
+    }
+    @ <tr><td>%s(zFlag)%h(aMime[i].zSuffix)<td>%h(aMime[i].zMimetype)</tr>
   }
   @ </tbody></table>
   style_table_sorter();
@@ -509,15 +652,49 @@ int doc_load_content(int vid, const char *zName, Blob *pContent){
   return rid;
 }
 
+/*
+** Check to verify that z[i] is contained within HTML markup.
+**
+** This works by looking backwards in the string for the most recent
+** '<' or '>' character.  If a '<' is found first, then we assume that
+** z[i] is within markup.  If a '>' is seen or neither character is seen,
+** then z[i] is not within markup.
+*/
+static int isWithinHtmlMarkup(const char *z, int i){
+  while( i>=0 && z[i]!='>' && z[i]!='<' ){ i--; }
+  return z[i]=='<';
+}
+
+/*
+** Check to see if z[i] is contained within an href='...' of markup.
+*/
+static int isWithinHref(const char *z, int i){
+  while( i>5
+     && !fossil_isspace(z[i])
+     && z[i]!='\'' && z[i]!='"'
+     && z[i]!='>'
+  ){ i--; }
+  if( i<=6 ) return 0;
+  if( z[i]!='\'' && z[i]!='\"' ) return 0;
+  if( strncmp(&z[i-5],"href=",5)!=0 ) return 0;
+  if( !fossil_isspace(z[i-6]) ) return 0;
+  return 1;
+}
+
 /*
 ** Transfer content to the output.  During the transfer, when text of
 ** the following form is seen:
 **
-**       href="$ROOT/
-**       action="$ROOT/
+**       href="$ROOT/..."
+**       action="$ROOT/..."
+**       href=".../doc/$CURRENT/..."
 **
-** Convert $ROOT to the root URI of the repository.  Allow ' in place of "
-** and any case for href or action.
+** Convert $ROOT to the root URI of the repository, and $CURRENT to the 
+** version number of the /doc/ document currently being displayed (if any).
+** Allow ' in place of " and any case for href or action.  
+**
+** Efforts are made to limit this translation to cases where the text is
+** fully contained with an HTML markup element.
 */
 void convert_href_and_output(Blob *pIn){
   int i, base;
@@ -530,10 +707,23 @@ void convert_href_and_output(Blob *pIn){
      && i-base>=9
      && ((fossil_strnicmp(&z[i-6],"href=",5)==0 && fossil_isspace(z[i-7])) ||
          (fossil_strnicmp(&z[i-8],"action=",7)==0 && fossil_isspace(z[i-9])) )
+     && isWithinHtmlMarkup(z, i-6)
     ){
       blob_append(cgi_output_blob(), &z[base], i-base);
       blob_appendf(cgi_output_blob(), "%R");
       base = i+5;
+    }else
+    if( z[i]=='$'
+     && strncmp(&z[i-5],"/doc/$CURRENT/", 11)==0
+     && isWithinHref(z,i-5)
+     && isWithinHtmlMarkup(z, i-5)
+     && strncmp(g.zPath, "doc/",4)==0
+    ){
+      int j;
+      for(j=7; g.zPath[j] && g.zPath[j]!='/'; j++){}
+      blob_append(cgi_output_blob(), &z[base], i-base);
+      blob_appendf(cgi_output_blob(), "%.*s", j-4, g.zPath+4);
+      base = i+8;
     }
   }
   blob_append(cgi_output_blob(), &z[base], i-base);
@@ -608,6 +798,7 @@ void document_render(
     }
 #endif
   }else{
+    fossil_free(style_csp(1));
     cgi_set_content_type(zMime);
     cgi_set_content(pBody);
   }
@@ -621,7 +812,7 @@ void document_render(
 ** URL: /doc/CHECKIN/FILE
 **
 ** CHECKIN can be either tag or hash prefix or timestamp identifying a
-** particular check, or the name of a branch (meaning the most recent
+** particular check-in, or the name of a branch (meaning the most recent
 ** check-in on that branch) or one of various magic words:
 **
 **     "tip"      means the most recent check-in
@@ -629,10 +820,14 @@ void document_render(
 **     "ckout"    means the current check-out, if the server is run from
 **                within a check-out, otherwise it is the same as "tip"
 **
+**     "latest"   means use the most recent check-in for the document
+**                regardless of what branch it occurs on.
+**
 ** FILE is the name of a file to delivered up as a webpage.  FILE is relative
 ** to the root of the source tree of the repository. The FILE must
 ** be a part of CHECKIN, except when CHECKIN=="ckout" when FILE is read
-** directly from disk and need not be a managed file.
+** directly from disk and need not be a managed file.  For /uv, FILE
+** can also be the hash of the unversioned file.
 **
 ** The "ckout" CHECKIN is intended for development - to provide a mechanism
 ** for looking at what a file will look like using the /doc webpage after
@@ -698,6 +893,16 @@ void doc_page(void){
       zCheckin = mprintf("%.*s", i, zName);
       if( fossil_strcmp(zCheckin,"ckout")==0 && g.localOpen==0 ){
         zCheckin = "tip";
+      }else if( fossil_strcmp(zCheckin,"latest")==0 ){
+        char *zNewCkin = db_text(0,
+          "SELECT uuid FROM blob, mlink, event, filename"
+          " WHERE filename.name=%Q"
+          "   AND mlink.fnid=filename.fnid"
+          "   AND blob.rid=mlink.mid"
+          "   AND event.objid=mlink.mid"
+          " ORDER BY event.mtime DESC LIMIT 1",
+          zName + i + 1);
+        if( zNewCkin ) zCheckin = zNewCkin;
       }
     }
     if( nMiss==count(azSuffix) ){
@@ -729,18 +934,22 @@ void doc_page(void){
     }
     if( isUV ){
       if( db_table_exists("repository","unversioned") ){
-        Stmt q;
-        db_prepare(&q, "SELECT hash, mtime FROM unversioned"
-                       " WHERE name=%Q", zName);
-        if( db_step(&q)==SQLITE_ROW ){
-          etag_check(ETAG_HASH, db_column_text(&q,0));
-          etag_last_modified(db_column_int64(&q,1));
-        }
-        db_finalize(&q);
-        if( unversioned_content(zName, &filebody)==0 ){
-          rid = 1;
-          zDfltTitle = zName;
+        rid = unversioned_content(zName, &filebody);
+        if( rid==1 ){
+          Stmt q;
+          db_prepare(&q, "SELECT hash, mtime FROM unversioned"
+                         " WHERE name=%Q", zName);
+          if( db_step(&q)==SQLITE_ROW ){
+            etag_check(ETAG_HASH, db_column_text(&q,0));
+            etag_last_modified(db_column_int64(&q,1));
+          }
+          db_finalize(&q);
+        }else if( rid==2 ){
+          zName = db_text(zName,
+             "SELECT name FROM unversioned WHERE hash=%Q", zName);
+          g.isConst = 1;
         }
+        zDfltTitle = zName;
       }
     }else if( fossil_strcmp(zCheckin,"ckout")==0 ){
       /* Read from the local checkout */
@@ -922,6 +1131,35 @@ void background_page(void){
 }
 
 
+/*
+** WEBPAGE: favicon.ico
+**
+** Return the configured "favicon.ico" image.  If no "favicon.ico" image
+** is defined, the returned image is for the Fossil lizard icon.
+**
+** The intended use case here is to supply an icon for the "fossil ui"
+** command.  For a permanent website, the recommended process is for
+** the admin to set up a project-specific icon and reference that icon
+** in the HTML header using a line like:
+**
+**   <link rel="icon" href="URL-FOR-YOUR-ICON" type="MIMETYPE"/>
+** 
+*/
+void favicon_page(void){
+  Blob icon;
+  char *zMime;
+
+  etag_check(ETAG_CONFIG, 0);
+  zMime = db_get("icon-mimetype", "image/gif");
+  blob_zero(&icon);
+  db_blob(&icon, "SELECT value FROM config WHERE name='icon-image'");
+  if( blob_size(&icon)==0 ){
+    blob_init(&icon, (char*)aLogo, sizeof(aLogo));
+  }
+  cgi_set_content_type(zMime);
+  cgi_set_content(&icon);
+}
+
 /*
 ** WEBPAGE: docsrch
 **
