@@ -65,7 +65,7 @@ void hyperlink_to_event_tagid(int tagid){
 */
 void event_page(void){
   int rid = 0;             /* rid of the event artifact */
-  char *zUuid;             /* UUID corresponding to rid */
+  char *zUuid;             /* artifact hash corresponding to rid */
   const char *zId;         /* Event identifier */
   const char *zVerbose;    /* Value of verbose option */
   char *zETime;            /* Time of the tech-note */
@@ -359,6 +359,10 @@ int event_commit_common(
 **    mimetype=TEXT     Mimetype for w= text
 **    newclr            Use a background color
 **    clr=TEXT          Background color to use if newclr
+**
+** For GET requests, when editing an existing technote newclr and clr
+** are implied if a custom color has been set on the previous version
+** of the technote.
 */
 void eventedit_page(void){
   char *zTag;
@@ -416,6 +420,15 @@ void eventedit_page(void){
   /* Figure out the color */
   if( rid ){
     zClr = db_text("", "SELECT bgcolor FROM event WHERE objid=%d", rid);
+    if( zClr && zClr[0] ){
+      const char * zRequestMethod = P("REQUEST_METHOD");
+      if(zRequestMethod && 'G'==zRequestMethod[0]){
+        /* Apply saved color by defaut for GET requests
+        ** (e.g., an Edit menu link).
+        */
+        zClrFlag = " checked";
+      }
+    }
   }else{
     zClr = "";
     isNew = 1;
@@ -497,6 +510,7 @@ void eventedit_page(void){
     @ <blockquote>
     blob_init(&event, 0, 0);
     blob_append(&event, zBody, -1);
+    safe_html_context(DOCSRC_WIKI);
     wiki_render_by_mimetype(&event, zMimetype);
     @ </blockquote><hr />
     blob_reset(&event);
