@@ -296,7 +296,7 @@ void symlink_copy(const char *zFrom, const char *zTo){
 */
 int file_perm(const char *zFilename, int eFType){
 #if !defined(_WIN32)
-  if( !getStat(zFilename, RepoFILE) ){
+  if( !getStat(zFilename, eFType) ){
      if( S_ISREG(fx.fileStat.st_mode) && ((S_IXUSR)&fx.fileStat.st_mode)!=0 )
       return PERM_EXE;
     else if( db_allow_symlinks() && S_ISLNK(fx.fileStat.st_mode) )
@@ -326,6 +326,78 @@ int file_islink(const char *zFilename){
   return file_perm(zFilename, RepoFILE)==PERM_LNK;
 }
 
+/*
+** Check every sub-directory of zRoot along the path to zFile.
+** If any sub-directory is really an ordinary file or a symbolic link,
+** return an integer which is the length of the prefix of zFile which
+** is the name of that object.  Return 0 if all no non-directory
+** objects are found along the path.
+**
+** Example:  Given inputs
+**
+**     zRoot = /home/alice/project1
+**     zFile = /home/alice/project1/main/src/js/fileA.js
+**
+** Look for objects in the following order:
+**
+**      /home/alice/project/main
+**      /home/alice/project/main/src
+**      /home/alice/project/main/src/js
+**
+** If any of those objects exist and are something other than a directory
+** then return the length of the name of the first non-directory object
+** seen.
+*/
+int file_nondir_objects_on_path(const char *zRoot, const char *zFile){
+  int i = (int)strlen(zRoot);
+  char *z = fossil_strdup(zFile);
+  assert( fossil_strnicmp(zRoot, z, i)==0 );
+  if( i && zRoot[i-1]=='/' ) i--;
+  while( z[i]=='/' ){
+    int j, rc;
+    for(j=i+1; z[j] && z[j]!='/'; j++){}
+    if( z[j]!='/' ) break;
+    z[j] = 0;
+    rc = file_isdir(z, SymFILE);
+    if( rc!=1 ){
+      if( rc==2 ){
+        fossil_free(z);
+        return j;
+      }
+      break;
+    }
+    z[j] = '/';
+    i = j;
+  }
+  fossil_free(z);
+  return 0;
+}
+
+/*
+** The file named zFile is suppose to be an in-tree file.  Check to
+** ensure that it will be safe to write to this file by verifying that
+** there are no symlinks or other non-directory objects in between the
+** root of the checkout and zFile.
+**
+** If a problem is found, print a warning message (using fossil_warning())
+** and return non-zero.  If everything is ok, return zero.
+*/
+int file_unsafe_in_tree_path(const char *zFile){
+  int n;
+  if( !file_is_absolute_path(zFile) ){
+    fossil_panic("%s is not an absolute pathname",zFile);
+  }
+  if( fossil_strnicmp(g.zLocalRoot, zFile, (int)strlen(g.zLocalRoot)) ){
+    fossil_panic("%s is not a prefix of %s", g.zLocalRoot, zFile);
+  }
+  n = file_nondir_objects_on_path(g.zLocalRoot, zFile);
+  if( n ){
+    fossil_warning("cannot write to %s because non-directory object %.*s"
+                   " is in the way", zFile, n, zFile);
+  }
+  return n;
+}
+
 /*
 ** Return 1 if zFilename is a directory.  Return 0 if zFilename
 ** does not exist.  Return 2 if zFilename exists but is something
@@ -349,6 +421,46 @@ int file_isdir(const char *zFilename, int eFType){
   return rc;
 }
 
+/*
+** Return true (1) if zFilename seems like it seems like a valid
+** repository database.
+*/
+int file_is_repository(const char *zFilename){
+  i64 sz;
+  sqlite3 *db = 0;
+  sqlite3_stmt *pStmt = 0;
+  int rc;
+  int i;
+  static const char *azReqTab[] = {
+     "blob", "delta", "rcvfrom", "user", "config"
+  };
+  if( !file_isfile(zFilename, ExtFILE) ) return 0;
+  sz = file_size(zFilename, ExtFILE);
+  if( sz<35328 ) return 0;
+  if( sz%512!=0 ) return 0;
+  rc = sqlite3_open_v2(zFilename, &db, 
+          SQLITE_OPEN_READWRITE, 0);
+  if( rc!=0 ) goto not_a_repo;
+  for(i=0; i<count(azReqTab); i++){
+    if( sqlite3_table_column_metadata(db, "main", azReqTab[i],0,0,0,0,0,0) ){
+      goto not_a_repo;
+    }
+  }
+  rc = sqlite3_prepare_v2(db, "SELECT 1 FROM config WHERE name='project-code'",
+                          -1, &pStmt, 0);
+  if( rc ) goto not_a_repo;
+  rc = sqlite3_step(pStmt);
+  if( rc!=SQLITE_ROW ) goto not_a_repo;
+  sqlite3_finalize(pStmt);
+  sqlite3_close(db);
+  return 1;
+
+not_a_repo:
+  sqlite3_finalize(pStmt);
+  sqlite3_close(db);
+  return 0;
+}
+
 
 /*
 ** Wrapper around the access() system call.
@@ -501,6 +613,7 @@ void file_copy(const char *zFrom, const char *zTo){
   }
   fclose(in);
   fclose(out);
+  if( file_isexe(zFrom, ExtFILE) ) file_setexe(zTo, 1);
 }
 
 /*
@@ -531,7 +644,10 @@ int file_setexe(const char *zFilename, int onoff){
   int rc = 0;
 #if !defined(_WIN32)
   struct stat buf;
-  if( fossil_stat(zFilename, &buf, RepoFILE)!=0 || S_ISLNK(buf.st_mode) ){
+  if( fossil_stat(zFilename, &buf, RepoFILE)!=0 
+   || S_ISLNK(buf.st_mode)
+   || S_ISDIR(buf.st_mode)
+  ){
     return 0;
   }
   if( onoff ){
@@ -928,6 +1044,7 @@ int file_simplify_name(char *z, int n, int slash){
   int i = 1, j;
   assert( z!=0 );
   if( n<0 ) n = strlen(z);
+  if( n==0 ) return 0;
 
   /* On windows and cygwin convert all \ characters to /
    * and remove extended path prefix if present */
@@ -1065,6 +1182,8 @@ int file_is_absolute_path(const char *zPath){
 ** Convert /A/../ to just /
 ** If the slash parameter is non-zero, the trailing slash, if any,
 ** is retained.
+**
+** See also: file_canonical_name_dup()
 */
 void file_canonical_name(const char *zOrigName, Blob *pOut, int slash){
   blob_zero(pOut);
@@ -1101,6 +1220,80 @@ void file_canonical_name(const char *zOrigName, Blob *pOut, int slash){
                                        blob_size(pOut), slash));
 }
 
+/*
+** Compute the canonical name of a file.  Store that name in
+** memory obtained from fossil_malloc() and return a pointer to the
+** name.
+**
+** See also: file_canonical_name()
+*/
+char *file_canonical_name_dup(const char *zOrigName){
+  Blob x;
+  if( zOrigName==0 ) return 0;
+  blob_init(&x, 0, 0);
+  file_canonical_name(zOrigName, &x, 0);
+  return blob_str(&x);
+}
+
+/*
+** The input is the name of an executable, such as one might
+** type on a command-line.  This routine resolves that name into
+** a full pathname.  The result is obtained from fossil_malloc()
+** and should be freed by the caller.
+**
+** This routine only works on unix.  On Windows, simply return
+** a copy of the input.
+*/
+char *file_fullexename(const char *zCmd){
+#ifdef _WIN32
+  return fossil_strdup(zCmd);
+#else
+  char *zPath;
+  char *z;
+  if( zCmd[0]=='/' ){
+    return fossil_strdup(zCmd);
+  }
+  if( strchr(zCmd,'/')!=0 ){
+    Blob out = BLOB_INITIALIZER;
+    file_canonical_name(zCmd, &out, 0);
+    z = fossil_strdup(blob_str(&out));
+    blob_reset(&out);
+    return z;
+  }
+  zPath = fossil_getenv("PATH");
+  while( zPath && zPath[0] ){
+    int n;
+    char *zColon;
+    zColon = strchr(zPath, ':');
+    n = zColon ? (int)(zColon-zPath) : (int)strlen(zPath);
+    z = mprintf("%.*s/%s", n, zPath, zCmd);
+    if( file_isexe(z, ExtFILE) ){
+      return z;
+    }
+    fossil_free(z);
+    if( zColon==0 ) break;
+    zPath = zColon+1;
+  }
+  return fossil_strdup(zCmd);
+#endif
+}
+
+/*
+** COMMAND: test-which
+**
+** Usage: %fossil test-which ARGS...
+**
+** For each argument, search the PATH for the executable with the name
+** and print its full pathname.
+*/
+void test_which_cmd(void){
+  int i;
+  for(i=2; i<g.argc; i++){
+    char *z = file_fullexename(g.argv[i]);
+    fossil_print("%z\n", z);
+  }
+}
+
 /*
 ** Emits the effective or raw stat() information for the specified
 ** file or directory, optionally preserving the trailing slash and
@@ -1127,6 +1320,7 @@ static void emitFileStat(
   fossil_print("  stat_rc                = %d\n", rc);
   sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", testFileStat.st_size);
   fossil_print("  stat_size              = %s\n", zBuf);
+  if( g.db==0 ) sqlite3_open(":memory:", &g.db);
   z = db_text(0, "SELECT datetime(%lld, 'unixepoch')", testFileStat.st_mtime);
   sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld (%s)", testFileStat.st_mtime, z);
   fossil_free(z);
@@ -1167,6 +1361,7 @@ static void emitFileStat(
   fossil_print("  file_islink            = %d\n", file_islink(zPath));
   fossil_print("  file_isexe(RepoFILE)   = %d\n", file_isexe(zPath,RepoFILE));
   fossil_print("  file_isdir(RepoFILE)   = %d\n", file_isdir(zPath,RepoFILE));
+  fossil_print("  file_is_repository     = %d\n", file_is_repository(zPath));
   if( reset ) resetStat();
 }
 
@@ -1193,7 +1388,7 @@ void cmd_test_file_environment(void){
   if( find_option("open-config", 0, 0)!=0 ){
     Th_OpenConfig(1);
   }
-  db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
+  db_find_and_open_repository(OPEN_ANY_SCHEMA|OPEN_OK_NOT_FOUND, 0);
   fossil_print("filenames_are_case_sensitive() = %d\n",
                filenames_are_case_sensitive());
   fossil_print("db_allow_symlinks_by_default() = %d\n",
@@ -1787,7 +1982,7 @@ int fossil_clearenv(void){
   return rc;
 #else
   extern char **environ;
-  environ = 0;
+  environ[0] = 0;
   return 0;
 #endif
 }
@@ -1811,6 +2006,57 @@ FILE *fossil_fopen(const char *zName, const char *zMode){
   return f;
 }
 
+/*
+** Works like fclose() except that:
+**
+** 1) is a no-op if f is 0 or if it is stdin.
+**
+** 2) If f is one of (stdout, stderr), it is flushed but not closed.
+*/
+void fossil_fclose(FILE *f){
+  if(f!=0){
+    if(stdout==f || stderr==f){
+      fflush(f);
+    }else if(stdin!=f){
+      fclose(f);
+    }
+  }
+}
+
+/*
+**   Works like fopen(zName,"wb") except that:
+**
+**   1) If zName is "-", the stdout handle is returned.
+**
+**   2) Else file_mkfolder() is used to create all directories
+**      which lead up to the file before opening it.
+**
+**   3) It fails fatally if the file cannot be opened.
+*/
+FILE *fossil_fopen_for_output(const char *zFilename){
+  if(zFilename[0]=='-' && zFilename[1]==0){
+    return stdout;
+  }else{
+    FILE * p;
+    file_mkfolder(zFilename, ExtFILE, 1, 0);
+    p = fossil_fopen(zFilename, "wb");
+    if( p==0 ){
+#if _WIN32
+      const char *zReserved = file_is_win_reserved(zFilename);
+      if( zReserved ){
+        fossil_fatal("cannot open \"%s\" because \"%s\" is "
+                     "a reserved name on Windows", zFilename,
+                     zReserved);
+      }
+#endif
+      fossil_fatal("unable to open file \"%s\" for writing",
+                   zFilename);
+    }
+    return p;
+  }
+}
+
+
 /*
 ** Return non-NULL if zFilename contains pathname elements that
 ** are reserved on Windows.  The returned string is the disallowed
@@ -2228,3 +2474,86 @@ void touch_cmd(){
     fossil_print("Touched %d file(s)\n", changeCount);
   }
 }
+
+/*
+** Returns non-zero if the specified file name ends with any reserved name,
+** e.g.: _FOSSIL_ or .fslckout.  Specifically, it returns 1 for exact match
+** or 2 for a tail match on a longer file name.
+**
+** For the sake of efficiency, zFilename must be a canonical name, e.g. an
+** absolute path using only forward slash ('/') as a directory separator.
+**
+** nFilename must be the length of zFilename.  When negative, strlen() will
+** be used to calculate it.
+*/
+int file_is_reserved_name(const char *zFilename, int nFilename){
+  const char *zEnd;  /* one-after-the-end of zFilename */
+  int gotSuffix = 0; /* length of suffix (-wal, -shm, -journal) */
+
+  assert( zFilename && "API misuse" );
+  if( nFilename<0 ) nFilename = (int)strlen(zFilename);
+  if( nFilename<8 ) return 0; /* strlen("_FOSSIL_") */
+  zEnd = zFilename + nFilename;
+  if( nFilename>=12 ){ /* strlen("_FOSSIL_-(shm|wal)") */
+    /* Check for (-wal, -shm, -journal) suffixes, with an eye towards
+    ** runtime speed. */
+    if( zEnd[-4]=='-' ){
+      if( fossil_strnicmp("wal", &zEnd[-3], 3)
+       && fossil_strnicmp("shm", &zEnd[-3], 3) ){
+        return 0;
+      }
+      gotSuffix = 4;
+    }else if( nFilename>=16 && zEnd[-8]=='-' ){ /*strlen(_FOSSIL_-journal) */
+      if( fossil_strnicmp("journal", &zEnd[-7], 7) ) return 0;
+      gotSuffix = 8;
+    }
+    if( gotSuffix ){
+      assert( 4==gotSuffix || 8==gotSuffix );
+      zEnd -= gotSuffix;
+      nFilename -= gotSuffix;
+      gotSuffix = 1;
+    }
+    assert( nFilename>=8 && "strlen(_FOSSIL_)" );
+    assert( gotSuffix==0 || gotSuffix==1 );
+  }
+  switch( zEnd[-1] ){
+    case '_':{
+      if( fossil_strnicmp("_FOSSIL_", &zEnd[-8], 8) ) return 0;
+      if( 8==nFilename ) return 1;
+      return zEnd[-9]=='/' ? 2 : gotSuffix;
+    }
+    case 'T':
+    case 't':{
+      if( nFilename<9 || zEnd[-9]!='.'
+       || fossil_strnicmp(".fslckout", &zEnd[-9], 9) ){
+        return 0; 
+      }
+      if( 9==nFilename ) return 1;
+      return zEnd[-10]=='/' ? 2 : gotSuffix;
+    }
+    default:{
+      return 0;
+    }
+  }
+}
+
+/*
+** COMMAND: test-is-reserved-name
+**
+** Usage: %fossil test-is-ckout-db FILENAMES...
+**
+** Passes each given name to file_is_reserved_name() and outputs one
+** line per file: the result value of that function followed by the
+** name.
+*/
+void test_is_reserved_name_cmd(void){
+  int i;
+
+  if(g.argc<3){
+    usage("FILENAME_1 [...FILENAME_N]");
+  }
+  for( i = 2; i < g.argc; ++i ){
+    const int check = file_is_reserved_name(g.argv[i], -1);
+    fossil_print("%d %s\n", check, g.argv[i]);
+  }
+}
