@@ -56,7 +56,7 @@
 **                        >22 or 0 (= no limit, resulting in a single line per
 **                        entry).
 **
-** See also: artifact, cat, descendants, info, leaves
+** See also: [[artifact]], [[cat]], [[descendants]], [[info]], [[leaves]]
 */
 void finfo_cmd(void){
   db_must_be_within_tree();
@@ -201,7 +201,7 @@ void finfo_cmd(void){
     );
     blob_zero(&line);
     if( iBrief ){
-      fossil_print("History of %s\n", blob_str(&fname));
+      fossil_print("History for %s\n", blob_str(&fname));
     }
     while( db_step(&q)==SQLITE_ROW ){
       const char *zFileUuid = db_column_text(&q, 0);
@@ -247,7 +247,7 @@ void finfo_cmd(void){
 **    -R|--repository FILE       Extract artifacts from repository FILE
 **    -r VERSION                 The specific check-in containing the file
 **
-** See also: finfo
+** See also: [[finfo]]
 */
 void cat_cmd(void){
   int i;
@@ -286,8 +286,8 @@ void cat_cmd(void){
 **    n=NUM      Show the first NUM changes only
 **    brbg       Background color by branch name
 **    ubg        Background color by user name
-**    ci=UUID    Ancestors of a particular check-in
-**    orig=UUID  If both ci and orig are supplied, only show those
+**    ci=HASH    Ancestors of a particular check-in
+**    orig=HASH  If both ci and orig are supplied, only show those
 **                 changes on a direct path from orig to ci.
 **    showid     Show RID values for debugging
 **
@@ -298,7 +298,7 @@ void cat_cmd(void){
 */
 void finfo_page(void){
   Stmt q;
-  const char *zFilename;
+  const char *zFilename = PD("name","");
   char zPrevDate[20];
   const char *zA;
   const char *zB;
@@ -323,7 +323,12 @@ void finfo_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  style_header("File History");
+  fnid = db_int(0, "SELECT fnid FROM filename WHERE name=%Q", zFilename);
+  if( fnid==0 ){
+    style_header("No such file");
+  }else{
+    style_header("History for %s", zFilename);
+  }
   login_anonymous_available();
   tmFlags = timeline_ss_submenu();
   if( tmFlags & TIMELINE_COLUMNAR ){
@@ -342,9 +347,7 @@ void finfo_page(void){
   if( uBg ) url_add_parameter(&url, "ubg", 0);
   baseCheckin = name_to_rid_www("ci");
   zPrevDate[0] = 0;
-  zFilename = PD("name","");
   cookie_render();
-  fnid = db_int(0, "SELECT fnid FROM filename WHERE name=%Q", zFilename);
   if( fnid==0 ){
     @ No such file: %h(zFilename)
     style_footer();
@@ -370,9 +373,9 @@ void finfo_page(void){
     " coalesce(event.euser, event.user),"            /* User who made chng */
     " mlink.pid,"                                    /* Parent file rid */
     " mlink.fid,"                                    /* File rid */
-    " (SELECT uuid FROM blob WHERE rid=mlink.pid),"  /* Parent file uuid */
-    " blob.uuid,"                                    /* Current file uuid */
-    " (SELECT uuid FROM blob WHERE rid=mlink.mid),"  /* Check-in uuid */
+    " (SELECT uuid FROM blob WHERE rid=mlink.pid),"  /* Parent file hash */
+    " blob.uuid,"                                    /* Current file hash */
+    " (SELECT uuid FROM blob WHERE rid=mlink.mid),"  /* Check-in hash */
     " event.bgcolor,"                                /* Background color */
     " (SELECT value FROM tagxref WHERE tagid=%d AND tagtype>0"
                                 " AND tagxref.rid=mlink.mid)," /* Branchname */
@@ -437,8 +440,9 @@ void finfo_page(void){
     }else{
       blob_appendf(&title, "Ancestors of file ");
     }
-    blob_appendf(&title,"<a href='%R/finfo?name=%T'>%h</a>",
-                 zFilename, zFilename);
+    blob_appendf(&title,"%z%h</a>",
+                 href("%R/file?name=%T&ci=%!S", zFilename, zUuid),
+                 zFilename);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
     blob_append(&title, origCheckin ? " between " : " from ", -1);
     blob_appendf(&title, "check-in %z%S</a>", zLink, zUuid);
@@ -451,8 +455,8 @@ void finfo_page(void){
       fossil_free(zUuid);
     }
   }else{
-    blob_appendf(&title, "History of ");
-    hyperlinked_path(zFilename, &title, 0, "tree", "");
+    blob_appendf(&title, "History for ");
+    hyperlinked_path(zFilename, &title, 0, "tree", "", LINKPATH_FILE);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
   }
   if( uBg ){
@@ -526,7 +530,7 @@ void finfo_page(void){
       @ <tr>
     }
     @ <td class="timelineTime">\
-    @ %z(href("%R/artifact/%!S",zUuid))%s(zTime)</a></td>
+    @ %z(href("%R/file?name=%T&ci=%!S",zFilename,zCkin))%s(zTime)</a></td>
     @ <td class="timelineGraph"><div id="m%d(gidx)" class="tl-nodemark"></div>
     @ </td>
     if( zBgClr && zBgClr[0] ){
@@ -539,9 +543,9 @@ void finfo_page(void){
     }else{
       @ <span class='timeline%s(zStyle)Comment'>
       if( (tmFlags & TIMELINE_VERBOSE)!=0 && zUuid ){
-        hyperlink_to_uuid(zUuid);
+        hyperlink_to_version(zUuid);
         @ part of check-in \
-        hyperlink_to_uuid(zCkin);
+        hyperlink_to_version(zCkin);
       }
     }
     @ %W(zCom)</span>
@@ -563,7 +567,7 @@ void finfo_page(void){
     cgi_printf("<span class='timeline%sDetail'>", zStyle);
     if( tmFlags & (TIMELINE_COMPACT|TIMELINE_VERBOSE) ) cgi_printf("(");
     if( zUuid && (tmFlags & TIMELINE_VERBOSE)==0 ){
-      @ file:&nbsp;%z(href("%R/artifact/%!S",zUuid))[%S(zUuid)]</a>
+      @ file:&nbsp;%z(href("%R/file?name=%T&ci=%!S",zFilename,zCkin))[%S(zUuid)]</a>
       if( fShowId ){
         int srcId = delta_source_rid(frid);
         if( srcId>0 ){
@@ -574,7 +578,7 @@ void finfo_page(void){
       }
     }
     @ check-in:&nbsp;\
-    hyperlink_to_uuid(zCkin);
+    hyperlink_to_version(zCkin);
     if( fShowId ){
       @ (%d(fmid))
     }
@@ -623,6 +627,9 @@ void finfo_page(void){
       if( fpid>0 ){
         @ %z(href("%R/fdiff?v1=%!S&v2=%!S",zPUuid,zUuid))[diff]</a>
       }
+      if( fileedit_is_editable(zFilename) ){
+        @ %z(href("%R/fileedit?filename=%T&checkin=%!S",zFilename,zCkin))[edit]</a>
+      }
       @ </span></span>
     }
     if( fDebug & FINFO_DEBUG_MLINK ){
