@@ -60,6 +60,23 @@ struct ForumThread {
 };
 #endif /* INTERFACE */
 
+/*
+** Return true if the forum entry with the given rid has been
+** subsequently edited.
+*/
+int forum_rid_has_been_edited(int rid){
+  static Stmt q;
+  int res;
+  db_static_prepare(&q,
+     "SELECT 1 FROM forumpost A, forumpost B"
+     " WHERE A.fpid=$rid AND B.froot=A.froot AND B.fprev=$rid"
+  );
+  db_bind_int(&q, "$rid", rid);
+  res = db_step(&q)==SQLITE_ROW;
+  db_reset(&q);
+  return res;
+}
+
 /*
 ** Delete a complete ForumThread and all its entries.
 */
@@ -141,6 +158,7 @@ static ForumThread *forumthread_create(int froot, int computeHierarchy){
   ForumEntry *pEntry;
   Stmt q;
   int sid = 1;
+  Bag seen = Bag_INIT;
   pThread = fossil_malloc( sizeof(*pThread) );
   memset(pThread, 0, sizeof(*pThread));
   db_prepare(&q,
@@ -160,14 +178,20 @@ static ForumThread *forumthread_create(int froot, int computeHierarchy){
     pEntry->sid = sid++;
     pEntry->pPrev = pThread->pLast;
     pEntry->pNext = 0;
+    bag_insert(&seen, pEntry->fpid);
     if( pThread->pLast==0 ){
       pThread->pFirst = pEntry;
     }else{
       pThread->pLast->pNext = pEntry;
     }
+    if( pEntry->firt && !bag_find(&seen,pEntry->firt) ){
+      pEntry->firt = froot;
+      pEntry->mfirt = froot;
+    }
     pThread->pLast = pEntry;
   }
   db_finalize(&q);
+  bag_clear(&seen);
 
   /* Establish which entries are the latest edit.  After this loop
   ** completes, entries that have non-NULL pLeaf should not be
@@ -202,12 +226,41 @@ static ForumThread *forumthread_create(int froot, int computeHierarchy){
   return pThread;
 }
 
+/*
+** List all forum threads to standard output.
+*/
+static void forum_thread_list(void){
+  Stmt q;
+  db_prepare(&q,
+    " SELECT"
+    "  datetime(max(fmtime)),"
+    "  sum(fprev IS NULL),"
+    "  froot"
+    " FROM forumpost"
+    " GROUP BY froot"
+    " ORDER BY 1;"
+  );
+  fossil_print("    id  cnt    most recent post\n");
+  fossil_print("------ ---- -------------------\n");
+  while( db_step(&q)==SQLITE_ROW ){
+    fossil_print("%6d %4d %s\n",
+      db_column_int(&q, 2),
+      db_column_int(&q, 1),
+      db_column_text(&q, 0)
+    );
+  }
+  db_finalize(&q);
+}
+
 /*
 ** COMMAND: test-forumthread
 **
-** Usage: %fossil test-forumthread THREADID
+** Usage: %fossil test-forumthread [THREADID]
 **
-** Display a summary of all messages on a thread.
+** Display a summary of all messages on a thread THREADID.  If the
+** THREADID argument is omitted, then show a list of all threads.
+**
+** This command is intended for testing an analysis only.
 */
 void forumthread_cmd(void){
   int fpid;
@@ -218,6 +271,10 @@ void forumthread_cmd(void){
 
   db_find_and_open_repository(0,0);
   verify_all_options();
+  if( g.argc==2 ){
+    forum_thread_list();
+    return;
+  }
   if( g.argc!=3 ) usage("THREADID");
   zName = g.argv[2];
   fpid = symbolic_name_to_rid(zName, "f");
@@ -235,12 +292,14 @@ void forumthread_cmd(void){
   fossil_print("froot = %d\n", froot);
   pThread = forumthread_create(froot, 1);
   fossil_print("Chronological:\n");
-           /*   123456789 123456789 123456789 123456789 123456789 123456789  */
-  fossil_print("     fpid      firt     fprev     mfirt     pLeaf    nReply\n");
+  fossil_print(
+/* 0         1         2         3         4         5         6         7    */
+/*  123456789 123456789 123456789 123456789 123456789 123456789 123456789 123 */
+  " sid      fpid      firt     fprev     mfirt     pLeaf  nReply  hash\n");
   for(p=pThread->pFirst; p; p=p->pNext){
-    fossil_print("%9d %9d %9d %9d %9d %9d\n",
+    fossil_print("%4d %9d %9d %9d %9d %9d  %6d  %8.8s\n", p->sid,
        p->fpid, p->firt, p->fprev, p->mfirt, p->pLeaf ? p->pLeaf->fpid : 0,
-       p->nReply);
+       p->nReply, p->zUuid);
   }
   fossil_print("\nDisplay\n");
   for(p=pThread->pDisplay; p; p=p->pDisplay){
@@ -261,7 +320,8 @@ void forum_render(
   const char *zTitle,         /* The title.  Might be NULL for no title */
   const char *zMimetype,      /* Mimetype of the message */
   const char *zContent,       /* Content of the message */
-  const char *zClass          /* Put in a <div> if not NULL */
+  const char *zClass,         /* Put in a <div> if not NULL */
+  int bScroll                 /* Large message content scrolls if true */
 ){
   if( zClass ){
     @ <div class='%s(zClass)'>
@@ -275,10 +335,17 @@ void forum_render(
   }
   if( zContent && zContent[0] ){
     Blob x;
+    if( bScroll ){
+      @ <div class='forumPostBody'>
+    }else{
+      @ <div class='forumPostFullBody'>
+    }
     blob_init(&x, 0, 0);
     blob_append(&x, zContent, -1);
+    safe_html_context(DOCSRC_FORUM);
     wiki_render_by_mimetype(&x, zMimetype);
     blob_reset(&x);
+    @ </div>
   }else{
     @ <i>Deleted</i>
   }
@@ -305,18 +372,54 @@ static void generateTrustControls(Manifest *pPost){
   @ <input type="hidden" name="trustuser" value="%h(pPost->zUser)">
 }
 
+/*
+** Compute a display name from a login name.
+**
+** If the input login is found in the USER table, then check the USER.INFO
+** field to see if it has display-name followed by an email address.
+** If it does, that becomes the new display name.  If not, let the display
+** name just be the login.
+**
+** Space to hold the returned name is obtained from fossil_strdup() or
+** mprintf() and should be freed by the caller.
+*/
+char *display_name_from_login(const char *zLogin){
+  static Stmt q;
+  char *zResult;
+  db_static_prepare(&q,
+     "SELECT display_name(info) FROM user WHERE login=$login"
+  );
+  db_bind_text(&q, "$login", zLogin);
+  if( db_step(&q)==SQLITE_ROW && db_column_type(&q,0)==SQLITE_TEXT ){
+    const char *zDisplay = db_column_text(&q,0);
+    if( fossil_strcmp(zDisplay,zLogin)==0 ){
+      zResult = fossil_strdup(zLogin);
+    }else{
+      zResult = mprintf("%s (%s)", zDisplay, zLogin);
+    }
+  }else{
+    zResult = fossil_strdup(zLogin);
+  }
+  db_reset(&q);
+  return zResult;
+}
+
 /*
 ** Display all posts in a forum thread in chronological order
 */
-static void forum_display_chronological(int froot, int target){
+static void forum_display_chronological(int froot, int target, int bRawMode){
   ForumThread *pThread = forumthread_create(froot, 0);
   ForumEntry *p;
   int notAnon = login_is_individual();
+  char cMode = bRawMode ? 'r' : 'c';
   for(p=pThread->pFirst; p; p=p->pNext){
     char *zDate;
     Manifest *pPost;
     int isPrivate;        /* True for posts awaiting moderation */
     int sameUser;         /* True if author is also the reader */
+    const char *zUuid;
+    char *zDisplayName;   /* The display name */
+    int sid;
 
     pPost = manifest_get(p->fpid, CFTYPE_FORUM, 0);
     if( pPost==0 ) continue;
@@ -331,42 +434,59 @@ static void forum_display_chronological(int froot, int target){
       @ <h1>%h(pPost->zThreadTitle)</h1>
     }
     zDate = db_text(0, "SELECT datetime(%.17g)", pPost->rDate);
-    @ <p>(%d(p->sid)) By %h(pPost->zUser) on %h(zDate)
+    zDisplayName = display_name_from_login(pPost->zUser);
+    sid = p->pEdit ? p->pEdit->sid : p->sid;
+    @ <h3 class='forumPostHdr'>(%d(sid)) By %h(zDisplayName) on %h(zDate)
+    fossil_free(zDisplayName);
     fossil_free(zDate);
     if( p->pEdit ){
-      @ edit of %z(href("%R/forumpost/%S?t=c",p->pEdit->zUuid))\
+      @ edit of %z(href("%R/forumpost/%S?t=%c",p->pEdit->zUuid,cMode))\
       @ %d(p->pEdit->sid)</a>
     }
     if( g.perm.Debug ){
       @ <span class="debug">\
-      @ <a href="%R/artifact/%h(p->zUuid)">(artifact)</a></span>
+      @ <a href="%R/artifact/%h(p->zUuid)">(artifact-%d(p->fpid))</a></span>
     }
     if( p->firt ){
       ForumEntry *pIrt = p->pPrev;
       while( pIrt && pIrt->fpid!=p->firt ) pIrt = pIrt->pPrev;
       if( pIrt ){
-        @ in reply to %z(href("%R/forumpost/%S?t=c",pIrt->zUuid))\
+        @ in reply to %z(href("%R/forumpost/%S?t=%c",pIrt->zUuid,cMode))\
         @ %d(pIrt->sid)</a>
       }
     }
+    zUuid = p->zUuid;
     if( p->pLeaf ){
-      @ updated by %z(href("%R/forumpost/%S?t=c",p->pLeaf->zUuid))\
+      @ updated by %z(href("%R/forumpost/%S?t=%c",p->pLeaf->zUuid,cMode))\
       @ %d(p->pLeaf->sid)</a>
+      zUuid = p->pLeaf->zUuid;
     }
     if( p->fpid!=target ){
-      @ %z(href("%R/forumpost/%S?t=c",p->zUuid))[link]</a>
+      @ %z(href("%R/forumpost/%S?t=%c",zUuid,cMode))[link]</a>
+    }
+    if( !bRawMode ){
+      @ %z(href("%R/forumpost/%S?raw",zUuid))[source]</a>
     }
     isPrivate = content_is_private(p->fpid);
     sameUser = notAnon && fossil_strcmp(pPost->zUser, g.zLogin)==0;
+    @ </h3>
     if( isPrivate && !g.perm.ModForum && !sameUser ){
       @ <p><span class="modpending">Awaiting Moderator Approval</span></p>
     }else{
-      forum_render(0, pPost->zMimetype, pPost->zWiki, 0);
+      const char *zMimetype;
+      if( bRawMode ){
+        zMimetype = "text/plain";
+      }else if( p->pLeaf!=0 ){
+        zMimetype = "text/plain";
+      }else{
+        zMimetype = pPost->zMimetype;
+      }
+      forum_render(0, zMimetype, pPost->zWiki, 0, 1);
     }
     if( g.perm.WrForum && p->pLeaf==0 ){
       int sameUser = login_is_individual()
                      && fossil_strcmp(pPost->zUser, g.zLogin)==0;
-      @ <p><form action="%R/forumedit" method="POST">
+      @ <div><form action="%R/forumedit" method="POST">
       @ <input type="hidden" name="fpid" value="%s(p->zUuid)">
       if( !isPrivate ){
         /* Reply and Edit are only available if the post has already
@@ -387,7 +507,115 @@ static void forum_display_chronological(int froot, int target){
         ** person who originally submitted the post */
         @ <input type="submit" name="reject" value="Delete">
       }
-      @ </form></p>
+      @ </form></div>
+    }
+    manifest_destroy(pPost);
+    @ </div>
+  }
+
+  /* Undocumented "threadtable" query parameter causes thread table
+  ** to be displayed for debugging purposes.
+  */
+  if( PB("threadtable") ){
+    @ <hr>
+    @ <table border="1" cellpadding="3" cellspacing="0">
+    @ <tr><th>sid<th>fpid<th>firt<th>fprev<th>mfirt<th>pLeaf<th>nReply<th>hash
+    for(p=pThread->pFirst; p; p=p->pNext){
+      @ <tr><td>%d(p->sid)<td>%d(p->fpid)<td>%d(p->firt)\
+      @ <td>%d(p->fprev)<td>%d(p->mfirt)\
+      @ <td>%d(p->pLeaf?p->pLeaf->fpid:0)<td>%d(p->nReply)\
+      @ <td>%S(p->zUuid)</tr>
+    }
+    @ </table>
+  }
+
+  forumthread_delete(pThread);
+}
+/*
+** Display all the edit history of post "target".
+*/
+static void forum_display_history(int froot, int target, int bRawMode){
+  ForumThread *pThread = forumthread_create(froot, 0);
+  ForumEntry *p;
+  int notAnon = login_is_individual();
+  char cMode = bRawMode ? 'r' : 'c';
+  ForumEntry *pLeaf = 0;
+  int cnt = 0;
+  for(p=pThread->pFirst; p; p=p->pNext){
+    if( p->fpid==target ){
+      pLeaf = p->pLeaf ? p->pLeaf : p;
+      break;
+    }
+  }
+  for(p=pThread->pFirst; p; p=p->pNext){
+    char *zDate;
+    Manifest *pPost;
+    int isPrivate;        /* True for posts awaiting moderation */
+    int sameUser;         /* True if author is also the reader */
+    const char *zUuid;
+    char *zDisplayName;   /* The display name */
+
+    if( p->fpid!=pLeaf->fpid && p->pLeaf!=pLeaf ) continue;
+    cnt++;
+    pPost = manifest_get(p->fpid, CFTYPE_FORUM, 0);
+    if( pPost==0 ) continue;
+    @ <div id="forum%d(p->fpid)" class="forumTime">
+    zDate = db_text(0, "SELECT datetime(%.17g)", pPost->rDate);
+    zDisplayName = display_name_from_login(pPost->zUser);
+    @ <h3 class='forumPostHdr'>(%d(p->sid)) By %h(zDisplayName) on %h(zDate)
+    fossil_free(zDisplayName);
+    fossil_free(zDate);
+    if( g.perm.Debug ){
+      @ <span class="debug">\
+      @ <a href="%R/artifact/%h(p->zUuid)">(artifact-%d(p->fpid))</a></span>
+    }
+    if( p->firt && cnt==1 ){
+      ForumEntry *pIrt = p->pPrev;
+      while( pIrt && pIrt->fpid!=p->firt ) pIrt = pIrt->pPrev;
+      if( pIrt ){
+        @ in reply to %z(href("%R/forumpost/%S?t=%c",pIrt->zUuid,cMode))\
+        @ %d(pIrt->sid)</a>
+      }
+    }
+    zUuid = p->zUuid;
+    @ %z(href("%R/forumpost/%S?t=c",zUuid))[link]</a>
+    if( !bRawMode ){
+      @ %z(href("%R/forumpost/%S?raw",zUuid))[source]</a>
+    }
+    isPrivate = content_is_private(p->fpid);
+    sameUser = notAnon && fossil_strcmp(pPost->zUser, g.zLogin)==0;
+    @ </h3>
+    if( isPrivate && !g.perm.ModForum && !sameUser ){
+      @ <p><span class="modpending">Awaiting Moderator Approval</span></p>
+    }else{
+      forum_render(0, bRawMode?"text/plain":pPost->zMimetype, pPost->zWiki,
+                   0, 1);
+    }
+    if( g.perm.WrForum && p->pLeaf==0 ){
+      int sameUser = login_is_individual()
+                     && fossil_strcmp(pPost->zUser, g.zLogin)==0;
+      @ <div><form action="%R/forumedit" method="POST">
+      @ <input type="hidden" name="fpid" value="%s(p->zUuid)">
+      if( !isPrivate ){
+        /* Reply and Edit are only available if the post has already
+        ** been approved */
+        @ <input type="submit" name="reply" value="Reply">
+        if( g.perm.Admin || sameUser ){
+          @ <input type="submit" name="edit" value="Edit">
+          @ <input type="submit" name="nullout" value="Delete">
+        }
+      }else if( g.perm.ModForum ){
+        /* Provide moderators with moderation buttons for posts that
+        ** are pending moderation */
+        @ <input type="submit" name="approve" value="Approve">
+        @ <input type="submit" name="reject" value="Reject">
+        generateTrustControls(pPost);
+      }else if( sameUser ){
+        /* A post that is pending moderation can be deleted by the
+        ** person who originally submitted the post */
+        @ <input type="submit" name="reject" value="Delete">
+      }
+      @ </form></div>
     }
     manifest_destroy(pPost);
     @ </div>
@@ -423,6 +651,7 @@ static int forum_display_hierarchical(int froot, int target){
   for(p=pThread->pDisplay; p; p=p->pDisplay){
     int isPrivate;         /* True for posts awaiting moderation */
     int sameUser;          /* True if reader is also the poster */
+    char *zDisplayName;    /* User name to be displayed */
     pOPost = manifest_get(p->fpid, CFTYPE_FORUM, 0);
     if( p->pLeaf ){
       fpid = p->pLeaf->fpid;
@@ -440,17 +669,19 @@ static int forum_display_hierarchical(int froot, int target){
       @ <div id='forum%d(fpid)' class='forumHier%s(zSel)' \
       @ style='margin-left: %d((p->nIndent-1)*iIndentScale)ex;'>
     }
-    pPost = manifest_get(fpid, CFTYPE_FORUM, 0);
     if( pPost==0 ) continue;
     if( pPost->zThreadTitle ){
       @ <h1>%h(pPost->zThreadTitle)</h1>
     }
     zDate = db_text(0, "SELECT datetime(%.17g)", pOPost->rDate);
-    @ <p>(%d(p->pLeaf?p->pLeaf->sid:p->sid)) By %h(pOPost->zUser) on %h(zDate)
+    zDisplayName = display_name_from_login(pOPost->zUser);
+    @ <h3 class='forumPostHdr'>\
+    @ (%d(p->sid)) By %h(zDisplayName) on %h(zDate)
+    fossil_free(zDisplayName);
     fossil_free(zDate);
     if( g.perm.Debug ){
       @ <span class="debug">\
-      @ <a href="%R/artifact/%h(p->zUuid)">(artifact)</a></span>
+      @ <a href="%R/artifact/%h(p->zUuid)">(artifact-%d(p->fpid))</a></span>
     }
     if( p->pLeaf ){
       zDate = db_text(0, "SELECT datetime(%.17g)", pPost->rDate);
@@ -462,30 +693,34 @@ static int forum_display_hierarchical(int froot, int target){
       fossil_free(zDate);
       if( g.perm.Debug ){
         @ <span class="debug">\
-        @ <a href="%R/artifact/%h(p->pLeaf->zUuid)">(artifact)</a></span>
+        @ <a href="%R/artifact/%h(p->pLeaf->zUuid)">\
+        @ (artifact-%d(p->pLeaf->fpid))</a></span>
       }
+      @ %z(href("%R/forumpost/%S?t=y",p->zUuid))[history]</a>
       manifest_destroy(pOPost);
     }
     if( fpid!=target ){
       @ %z(href("%R/forumpost/%S",zUuid))[link]</a>
     }
+    @ %z(href("%R/forumpost/%S?raw",zUuid))[source]</a>
     if( p->firt ){
       ForumEntry *pIrt = p->pPrev;
-      while( pIrt && pIrt->fpid!=p->firt ) pIrt = pIrt->pPrev;
+      while( pIrt && pIrt->fpid!=p->mfirt ) pIrt = pIrt->pPrev;
       if( pIrt ){
         @ in reply to %z(href("%R/forumpost/%S?t=h",pIrt->zUuid))\
         @ %d(pIrt->sid)</a>
       }
     }
+    @ </h3>
     isPrivate = content_is_private(fpid);
     sameUser = notAnon && fossil_strcmp(pPost->zUser, g.zLogin)==0;
     if( isPrivate && !g.perm.ModForum && !sameUser ){
       @ <p><span class="modpending">Awaiting Moderator Approval</span></p>
     }else{
-      forum_render(0, pPost->zMimetype, pPost->zWiki, 0);
+      forum_render(0, pPost->zMimetype, pPost->zWiki, 0, 1);
     }
     if( g.perm.WrForum ){
-      @ <p><form action="%R/forumedit" method="POST">
+      @ <div><form action="%R/forumedit" method="POST">
       @ <input type="hidden" name="fpid" value="%s(zUuid)">
       if( !isPrivate ){
         /* Reply and Edit are only available if the post has already
@@ -506,7 +741,7 @@ static int forum_display_hierarchical(int froot, int target){
         ** person who originally submitted the post */
         @ <input type="submit" name="reject" value="Delete">
       }
-      @ </form></p>
+      @ </form></div>
     }
     manifest_destroy(pPost);
     @ </div>
@@ -515,6 +750,20 @@ static int forum_display_hierarchical(int froot, int target){
   return target;
 }
 
+/*
+** Emits all JS code required by /forumpost.
+*/
+static void forumpost_emit_page_js(){
+  static int once = 0;
+  if(0==once){
+    once = 1;
+    style_emit_script_fossil_bootstrap(1);
+    builtin_request_js("forum.js");
+    builtin_request_js("fossil.dom.js");
+    builtin_request_js("fossil.page.forumpost.js");
+  }
+}
+
 /*
 ** WEBPAGE: forumpost
 **
@@ -526,13 +775,39 @@ static int forum_display_hierarchical(int froot, int target){
 ** Query parameters:
 **
 **   name=X        REQUIRED.  The hash of the post to display
-**   t=MODE        Display mode. MODE is 'c' for chronological or
-**                   'h' for hierarchical, or 'a' for automatic.
+**   t=MODE        Display mode.
+**                   'c' for chronological
+**                   'h' for hierarchical
+**                   'a' for automatic
+**                   'r' for raw
+**                   'y' for history of post X only
+**   raw           If present, show only the post specified and
+**                 show its original unformatted source text.
 */
 void forumpost_page(void){
   forumthread_page();
 }
 
+/*
+** Add an appropriate style_header() to include title of the
+** given forum post.
+*/
+static int forumthread_page_header(int froot, int fpid){
+  char *zThreadTitle = 0;
+
+  zThreadTitle = db_text("",
+    "SELECT"
+    " substr(event.comment,instr(event.comment,':')+2)"
+    " FROM forumpost, event"
+    " WHERE event.objid=forumpost.fpid"
+    "   AND forumpost.fpid=%d;",
+    fpid
+  );
+  style_header("%s%s", zThreadTitle, zThreadTitle[0] ? "" : "Forum");
+  fossil_free(zThreadTitle);
+  return 0;
+}
+
 /*
 ** WEBPAGE: forumthread
 **
@@ -543,14 +818,20 @@ void forumpost_page(void){
 ** Query parameters:
 **
 **   name=X        REQUIRED.  The hash of any post of the thread.
-**   t=MODE        Display mode. MODE is 'c' for chronological or
-**                   'h' for hierarchical, or 'a' for automatic.
+**   t=MODE        Display mode. MODE is...
+**                   'c' for chronological, or
+**                   'h' for hierarchical, or
+**                   'a' for automatic, or
+**                   'r' for raw.
+**   raw           Show only the post given by name= and show it unformatted
+**   hist          Show only the edit history for the name= post
 */
 void forumthread_page(void){
   int fpid;
   int froot;
   const char *zName = P("name");
   const char *zMode = PD("t","a");
+  int bRaw = PB("raw");
   login_check_credentials();
   if( !g.perm.RdForum ){
     login_needed(g.anon.RdForum);
@@ -563,7 +844,6 @@ void forumthread_page(void){
   if( fpid<=0 ){
     webpage_error("Unknown or ambiguous forum id: \"%s\"", zName);
   }
-  style_header("Forum");
   froot = db_int(0, "SELECT froot FROM forumpost WHERE fpid=%d", fpid);
   if( froot==0 ){
     webpage_error("Not a forum post: \"%s\"", zName);
@@ -576,14 +856,43 @@ void forumthread_page(void){
       zMode = "h";
     }
   }
-  if( zMode[0]=='c' ){
+  if( zMode[0]!='y' ){
+    forumthread_page_header(froot, fpid);
+  }
+  if( bRaw && fpid ){
+    Manifest *pPost;
+    pPost = manifest_get(fpid, CFTYPE_FORUM, 0);
+    if( pPost==0 ){
+      @ <p>No such forum post: %h(zName)
+    }else{
+      int isPrivate = content_is_private(fpid);
+      int notAnon = login_is_individual();
+      int sameUser = notAnon && fossil_strcmp(pPost->zUser, g.zLogin)==0;
+      if( isPrivate && !g.perm.ModForum && !sameUser ){
+        @ <p><span class="modpending">Awaiting Moderator Approval</span></p>
+      }else{
+        forum_render(0, "text/plain", pPost->zWiki, 0, 0);
+      }
+      manifest_destroy(pPost);
+    }
+  }else if( zMode[0]=='c' ){
     style_submenu_element("Hierarchical", "%R/%s/%s?t=h", g.zPath, zName);
-    forum_display_chronological(froot, fpid);
+    style_submenu_element("Unformatted", "%R/%s/%s?t=r", g.zPath, zName);
+    forum_display_chronological(froot, fpid, 0);
+  }else if( zMode[0]=='r' ){
+    style_submenu_element("Chronological", "%R/%s/%s?t=c", g.zPath, zName);
+    style_submenu_element("Hierarchical", "%R/%s/%s?t=h", g.zPath, zName);
+    forum_display_chronological(froot, fpid, 1);
+  }else if( zMode[0]=='y' ){
+    style_header("Edit History Of A Forum Post");
+    style_submenu_element("Complete Thread", "%R/%s/%s?t=a", g.zPath, zName);
+    forum_display_history(froot, fpid, 1);
   }else{
     style_submenu_element("Chronological", "%R/%s/%s?t=c", g.zPath, zName);
+    style_submenu_element("Unformatted", "%R/%s/%s?t=r", g.zPath, zName);
     forum_display_hierarchical(froot, fpid);
   }
-  style_load_js("forum.js");
+  forumpost_emit_page_js();
   style_footer();
 }
 
@@ -597,6 +906,15 @@ static int forum_need_moderation(void){
   return 1;
 }
 
+/*
+** Return true if the string is white-space only.
+*/
+static int whitespace_only(const char *z){
+  if( z==0 ) return 1;
+  while( z[0] && fossil_isspace(z[0]) ){ z++; }
+  return z[0]==0;
+}
+
 /*
 ** Add a new Forum Post artifact to the repository.
 **
@@ -616,8 +934,12 @@ static int forum_post(
   int iBasis;
   Blob x, cksum, formatCheck, errMsg;
   Manifest *pPost;
+  int nContent = zContent ? (int)strlen(zContent) : 0;
 
   schema_forum();
+  if( iEdit==0 && whitespace_only(zContent) ){
+    return 0;
+  }
   if( iInReplyTo==0 && iEdit>0 ){
     iBasis = iEdit;
     iInReplyTo = db_int(0, "SELECT firt FROM forumpost WHERE fpid=%d", iEdit);
@@ -662,7 +984,7 @@ static int forum_post(
     }
   }
   blob_appendf(&x, "U %F\n", zUser);
-  blob_appendf(&x, "W %d\n%s\n", strlen(zContent), zContent);
+  blob_appendf(&x, "W %d\n%s\n", nContent, zContent);
   md5sum_blob(&x, &cksum);
   blob_appendf(&x, "Z %b\n", &cksum);
   blob_reset(&cksum);
@@ -686,7 +1008,9 @@ static int forum_post(
     blob_reset(&x);
     return 0;
   }else{
-    int nrid = wiki_put(&x, 0, forum_need_moderation());
+    int nrid = wiki_put(&x, iEdit>0 ? iEdit : 0,
+                        forum_need_moderation());
+    blob_reset(&x);
     cgi_redirectf("%R/forumpost/%S", rid_to_uuid(nrid));
     return 1;
   }
@@ -701,12 +1025,13 @@ static void forum_entry_widget(
   const char *zContent
 ){
   if( zTitle ){
-    @ Title: <input type="input" name="title" value="%h(zTitle)" size="50"><br>
+    @ Title: <input type="input" name="title" value="%h(zTitle)" size="50"
+    @ maxlength="125"><br>
   }
   @ %z(href("%R/markup_help"))Markup style</a>:
   mimetype_option_menu(zMimetype);
-  @ <br><textarea name="content" class="wikiedit" cols="80" \
-  @ rows="25" wrap="virtual">%h(zContent)</textarea><br>
+  @ <br><textarea aria-label="Content:" name="content" class="wikiedit" \
+  @ cols="80" rows="25" wrap="virtual">%h(zContent)</textarea><br>
 }
 
 /*
@@ -792,12 +1117,12 @@ void forumnew_page(void){
     login_needed(g.anon.WrForum);
     return;
   }
-  if( P("submit") ){
+  if( P("submit") && cgi_csrf_safe(1) ){
     if( forum_post(zTitle, 0, 0, 0, zMimetype, zContent) ) return;
   }
-  if( P("preview") ){
+  if( P("preview") && !whitespace_only(zContent) ){
     @ <h1>Preview:</h1>
-    forum_render(zTitle, zMimetype, zContent, "forumEdit");
+    forum_render(zTitle, zMimetype, zContent, "forumEdit", 1);
   }
   style_header("New Forum Thread");
   @ <form action="%R/forume1" method="POST">
@@ -805,13 +1130,14 @@ void forumnew_page(void){
   forum_from_line();
   forum_entry_widget(zTitle, zMimetype, zContent);
   @ <input type="submit" name="preview" value="Preview">
-  if( P("preview") ){
+  if( P("preview") && !whitespace_only(zContent) ){
     @ <input type="submit" name="submit" value="Submit">
   }else{
     @ <input type="submit" name="submit" value="Submit" disabled>
   }
   if( g.perm.Debug ){
-    /* For the test-forumnew page add these extra debugging controls */
+    /* Give extra control over the post to users with the special
+     * Debug capability, which includes Admin and Setup users */
     @ <div class="debug">
     @ <label><input type="checkbox" name="dryrun" %s(PCK("dryrun"))> \
     @ Dry run</label>
@@ -835,10 +1161,13 @@ void forumnew_page(void){
 */
 void forumedit_page(void){
   int fpid;
+  int froot;
   Manifest *pPost = 0;
+  Manifest *pRootPost = 0;
   const char *zMimetype = 0;
   const char *zContent = 0;
   const char *zTitle = 0;
+  char *zDate = 0;
   int isCsrfSafe;
   int isDelete = 0;
 
@@ -851,6 +1180,10 @@ void forumedit_page(void){
   if( fpid<=0 || (pPost = manifest_get(fpid, CFTYPE_FORUM, 0))==0 ){
     webpage_error("Missing or invalid fpid query parameter");
   }
+  froot = db_int(0, "SELECT froot FROM forumpost WHERE fpid=%d", fpid);
+  if( froot==0 || (pRootPost = manifest_get(froot, CFTYPE_FORUM, 0))==0 ){
+    webpage_error("fpid does not appear to be a forum post: \"%d\"", fpid);
+  }
   if( P("cancel") ){
     cgi_redirectf("%R/forumpost/%S",P("fpid"));
     return;
@@ -859,7 +1192,7 @@ void forumedit_page(void){
   if( g.perm.ModForum && isCsrfSafe ){
     if( P("approve") ){
       const char *zUserToTrust;
-      moderation_approve(fpid);
+      moderation_approve('f', fpid);
       if( g.perm.AdminForum
        && PB("trust")
        && (zUserToTrust = P("trustuser"))!=0
@@ -888,10 +1221,13 @@ void forumedit_page(void){
     }
   }
   isDelete = P("nullout")!=0;
-  if( P("submit") && isCsrfSafe ){
+  if( P("submit")
+   && isCsrfSafe
+   && (zContent = PDT("content",""))!=0
+   && (!whitespace_only(zContent) || isDelete)
+  ){
     int done = 1;
     const char *zMimetype = PD("mimetype",DEFAULT_FORUM_MIMETYPE);
-    const char *zContent = PDT("content","");
     if( P("reply") ){
       done = forum_post(0, fpid, 0, 0, zMimetype, zContent);
     }else if( P("edit") || isDelete ){
@@ -908,16 +1244,16 @@ void forumedit_page(void){
     style_header("Delete %s", zTitle ? "Post" : "Reply");
     @ <h1>Original Post:</h1>
     forum_render(pPost->zThreadTitle, pPost->zMimetype, pPost->zWiki,
-                 "forumEdit");
+                 "forumEdit", 1);
     @ <h1>Change Into:</h1>
-    forum_render(zTitle, zMimetype, zContent,"forumEdit");
+    forum_render(zTitle, zMimetype, zContent,"forumEdit", 1);
     @ <form action="%R/forume2" method="POST">
     @ <input type="hidden" name="fpid" value="%h(P("fpid"))">
     @ <input type="hidden" name="nullout" value="1">
     @ <input type="hidden" name="mimetype" value="%h(zMimetype)">
     @ <input type="hidden" name="content" value="%h(zContent)">
     if( zTitle ){
-      @ <input type="hidden" name="title" value="%h(zTitle)">
+      @ <input aria-label="Title" type="hidden" name="title" value="%h(zTitle)">
     }
   }else if( P("edit") ){
     /* Provide an edit to the fpid post */
@@ -930,14 +1266,14 @@ void forumedit_page(void){
       zTitle = fossil_strdup(pPost->zThreadTitle);
     }
     style_header("Edit %s", zTitle ? "Post" : "Reply");
-    @ <h1>Original Post:</h1>
+    @ <h2>Original Post:</h2>
     forum_render(pPost->zThreadTitle, pPost->zMimetype, pPost->zWiki,
-                 "forumEdit");
+                 "forumEdit", 1);
     if( P("preview") ){
-      @ <h1>Preview of Edited Post:</h1>
-      forum_render(zTitle, zMimetype, zContent,"forumEdit");
+      @ <h2>Preview of Edited Post:</h2>
+      forum_render(zTitle, zMimetype, zContent,"forumEdit", 1);
     }
-    @ <h1>Revised Message:</h1>
+    @ <h2>Revised Message:</h2>
     @ <form action="%R/forume2" method="POST">
     @ <input type="hidden" name="fpid" value="%h(P("fpid"))">
     @ <input type="hidden" name="edit" value="1">
@@ -945,16 +1281,25 @@ void forumedit_page(void){
     forum_entry_widget(zTitle, zMimetype, zContent);
   }else{
     /* Reply */
+    char *zDisplayName;
     zMimetype = PD("mimetype",DEFAULT_FORUM_MIMETYPE);
     zContent = PDT("content","");
     style_header("Reply");
-    @ <h1>Replying To:</h1>
-    forum_render(0, pPost->zMimetype, pPost->zWiki, "forumEdit");
-    if( P("preview") ){
-      @ <h1>Preview:</h1>
-      forum_render(0, zMimetype,zContent, "forumEdit");
+    if( pRootPost->zThreadTitle ){
+      @ <h1>Thread: %h(pRootPost->zThreadTitle)</h1>
     }
-    @ <h1>Enter Reply:</h1>
+    @ <h2>Replying To:</h2>
+    zDate = db_text(0, "SELECT datetime(%.17g)", pPost->rDate);
+    zDisplayName = display_name_from_login(pPost->zUser);
+    @ <h3 class='forumPostHdr'>By %h(zDisplayName) on %h(zDate)</h3>
+    fossil_free(zDisplayName);
+    fossil_free(zDate);
+    forum_render(0, pPost->zMimetype, pPost->zWiki, "forumEdit", 1);
+    if( P("preview") && !whitespace_only(zContent) ){
+      @ <h2>Preview:</h2>
+      forum_render(0, zMimetype,zContent, "forumEdit", 1);
+    }
+    @ <h2>Enter Reply:</h2>
     @ <form action="%R/forume2" method="POST">
     @ <input type="hidden" name="fpid" value="%h(P("fpid"))">
     @ <input type="hidden" name="reply" value="1">
@@ -965,7 +1310,7 @@ void forumedit_page(void){
     @ <input type="submit" name="preview" value="Preview">
   }
   @ <input type="submit" name="cancel" value="Cancel">
-  if( P("preview") || isDelete ){
+  if( (P("preview") && !whitespace_only(zContent)) || isDelete ){
     @ <input type="submit" name="submit" value="Submit">
   }
   if( g.perm.Debug ){
@@ -984,6 +1329,7 @@ void forumedit_page(void){
 }
 
 /*
+** WEBPAGE: forummain
 ** WEBPAGE: forum
 **
 ** The main page for the forum feature.  Show a list of recent forum
