@@ -58,6 +58,7 @@ struct GraphRow {
   i8 nParent;                 /* Number of parents. */
   i8 nCherrypick;             /* Subset of aParent that are cherrypicks */
   i8 nNonCherrypick;          /* Number of non-cherrypick parents */
+  u8 nMergeChild;             /* Number of merge children */
   int *aParent;               /* Array of parents.  0 element is primary .*/
   char *zBranch;              /* Branch name */
   char *zBgClr;               /* Background Color */
@@ -474,7 +475,8 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
   if( (tmFlags & (TIMELINE_DISJOINT|TIMELINE_XMERGE))!=0 ){
     for(pRow=p->pFirst; pRow; pRow=pRow->pNext){
       for(i=1; i<pRow->nParent; i++){
-        if( hashFind(p, pRow->aParent[i])==0 ){
+        GraphRow *pParent = hashFind(p, pRow->aParent[i]);
+        if( pParent==0 ){
           memmove(pRow->aParent+i, pRow->aParent+i+1, 
                   sizeof(pRow->aParent[0])*(pRow->nParent-i-1));
           pRow->nParent--;
@@ -488,6 +490,58 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
       }
     }
   }
+ 
+  /* Put the deepest (earliest) merge parent first in the list.
+  ** An off-screen merge parent is considered deepest.
+  */
+  for(pRow=p->pFirst; pRow; pRow=pRow->pNext ){
+    if( pRow->nParent<=1 ) continue;
+    for(i=1; i<pRow->nParent; i++){
+      GraphRow *pParent = hashFind(p, pRow->aParent[i]);
+      if( pParent ) pParent->nMergeChild++;
+    }
+    if( pRow->nCherrypick>1 ){
+      int iBest = -1;
+      int iDeepest = -1;
+      for(i=pRow->nNonCherrypick; i<pRow->nParent; i++){
+        GraphRow *pParent = hashFind(p, pRow->aParent[i]);
+        if( pParent==0 ){
+          iBest = i;
+          break;
+        }
+        if( pParent->idx>iDeepest ){
+          iDeepest = pParent->idx;
+          iBest = i;
+        }
+      }
+      i = pRow->nNonCherrypick;
+      if( iBest>i ){
+        int x = pRow->aParent[i];
+        pRow->aParent[i] = pRow->aParent[iBest];
+        pRow->aParent[iBest] = x;
+      }
+    }
+    if( pRow->nNonCherrypick>2 ){
+      int iBest = -1;
+      int iDeepest = -1;
+      for(i=1; i<pRow->nNonCherrypick; i++){
+        GraphRow *pParent = hashFind(p, pRow->aParent[i]);
+        if( pParent==0 ){
+          iBest = i;
+          break;
+        }
+        if( pParent->idx>iDeepest ){
+          iDeepest = pParent->idx;
+          iBest = i;
+        }
+      }
+      if( iBest>1 ){
+        int x = pRow->aParent[1];
+        pRow->aParent[1] = pRow->aParent[iBest];
+        pRow->aParent[iBest] = x;
+      }
+    }
+  }
 
   /* If the primary parent is in a different branch, but there are
   ** other parents in the same branch, reorder the parents to make
@@ -536,13 +590,15 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
   }
 
   if( tmFlags & TIMELINE_FILLGAPS ){
-    /* If a node has no pChild and there is a node higher up in the graph
+    /* If a node has no pChild in the graph
+    ** and there is a node higher up in the graph
     ** that is in the same branch and has no in-graph parent, then
     ** make the lower node a step-child of the upper node.  This will
     ** be represented on the graph by a thick dotted line without an arrowhead.
     */
     for(pRow=p->pFirst; pRow; pRow=pRow->pNext){
       if( pRow->pChild ) continue;
+      if( pRow->isLeaf ) continue;
       for(pLoop=pRow->pPrev; pLoop; pLoop=pLoop->pPrev){
         if( pLoop->nParent>0
          && pLoop->zBranch==pRow->zBranch
@@ -668,12 +724,26 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
   ** Insert merge rails and merge arrows
   */
   for(pRow=p->pFirst; pRow; pRow=pRow->pNext){
+    int iReuseIdx = -1;
+    int iReuseRail = -1;
+    int isCherrypick = 0;
     for(i=1; i<pRow->nParent; i++){
       int parentRid = pRow->aParent[i];
+      if( i==pRow->nNonCherrypick ){
+        /* Because full merges are laid out before cherrypicks,
+        ** it is ok to use a full-merge raise for a cherrypick.
+        ** See the graph on check-in 8ac66ef33b464d28 for example
+        **    iReuseIdx = -1;
+        **    iReuseRail = -1; */
+        isCherrypick = 1;
+      }
       pDesc = hashFind(p, parentRid);
       if( pDesc==0 ){
-        /* Merge from a node that is off-screen */
         int iMrail = -1;
+        /* Merge from a node that is off-screen */
+        if( iReuseIdx>=p->nRow+1 ){
+          continue;  /* Suppress multiple off-screen merges */
+        }
         for(j=0; j<GR_MAX_RAIL; j++){
           if( mergeRiserFrom[j]==parentRid ){
             iMrail = j;
@@ -685,6 +755,8 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
           if( p->mxRail>=GR_MAX_RAIL ) return;
           mergeRiserFrom[iMrail] = parentRid;
         }
+        iReuseIdx = p->nRow+1;
+        iReuseRail = iMrail;
         mask = BIT(iMrail);
         if( i>=pRow->nNonCherrypick ){
           pRow->mergeIn[iMrail] = 2;
@@ -697,9 +769,30 @@ void graph_finish(GraphContext *p, const char *zLeftBranch, u32 tmFlags){
           pLoop->railInUse |= mask;
         }
       }else{
-        /* Merge from an on-screen node */
-        createMergeRiser(p, pDesc, pRow, i>=pRow->nNonCherrypick);
-        if( p->mxRail>=GR_MAX_RAIL ) return;
+        /* The merge parent node does exist on this graph */
+        if( iReuseIdx>pDesc->idx
+         && pDesc->nMergeChild==1
+        ){
+          /* Reuse an existing merge riser */
+          pDesc->mergeOut = iReuseRail;
+          if( isCherrypick ){
+            pDesc->cherrypickUpto = pDesc->idx;
+          }else{
+            pDesc->hasNormalOutMerge = 1;
+            pDesc->mergeUpto = pDesc->idx;
+          }
+        }else{
+          /* Create a new merge for an on-screen node */
+          createMergeRiser(p, pDesc, pRow, isCherrypick);
+          if( p->mxRail>=GR_MAX_RAIL ) return;
+          if( iReuseIdx<0
+           && pDesc->nMergeChild==1
+           && (pDesc->iRail!=pDesc->mergeOut || pDesc->isLeaf)
+          ){
+            iReuseIdx = pDesc->idx;
+            iReuseRail = pDesc->mergeOut;
+          }
+        }
       }
     }
   }
