@@ -113,28 +113,6 @@ static void redirect_to_g(void){
   }
 }
 
-/*
-** The IP address of the client is stored as part of login cookies.
-** But some clients are behind firewalls that shift the IP address
-** with each HTTP request.  To allow such (broken) clients to log in,
-** extract just a prefix of the IP address.
-*/
-static char *ipPrefix(const char *zIP){
-  int i, j;
-  static int ip_prefix_terms = -1;
-  if( ip_prefix_terms<0 ){
-    ip_prefix_terms = db_get_int("ip-prefix-terms",2);
-  }
-  if( ip_prefix_terms==0 ) return mprintf("0");
-  for(i=j=0; zIP[i]; i++){
-    if( zIP[i]=='.' ){
-      j++;
-      if( j==ip_prefix_terms ) break;
-    }
-  }
-  return mprintf("%.*s", i, zIP);
-}
-
 /*
 ** Return an abbreviated project code.  The abbreviation is the first
 ** 16 characters of the project code.
@@ -285,39 +263,42 @@ char *login_gen_user_cookie_value(const char *zUsername, const char *zHash){
 ** If zDest is not NULL then the generated cookie is copied to
 ** *zDdest and ownership is transfered to the caller (who should
 ** eventually pass it to free()).
+**
+** If bSessionCookie is true, the cookie will be a session cookie,
+** else a persistent cookie. If it's a session cookie, the
+** [user].[cexpire] and [user].[cookie] entries will be modified as if
+** it were a persistent cookie because doing so is necessary for
+** fossil's own "is this cookie still valid?" checks to work.
 */
 void login_set_user_cookie(
   const char *zUsername,  /* User's name */
   int uid,                /* User's ID */
-  char **zDest            /* Optional: store generated cookie value. */
+  char **zDest,           /* Optional: store generated cookie value. */
+  int bSessionCookie      /* True for session-only cookie */
 ){
   const char *zCookieName = login_cookie_name();
   const char *zExpire = db_get("cookie-expire","8766");
-  int expires = atoi(zExpire)*3600;
-  char *zHash;
+  const int expires = atoi(zExpire)*3600;
+  char *zHash = 0;
   char *zCookie;
   const char *zIpAddr = PD("REMOTE_ADDR","nil"); /* IP address of user */
-  char *zRemoteAddr = ipPrefix(zIpAddr);         /* Abbreviated IP address */
 
   assert((zUsername && *zUsername) && (uid > 0) && "Invalid user data.");
   zHash = db_text(0,
       "SELECT cookie FROM user"
       " WHERE uid=%d"
-      "   AND ipaddr=%Q"
       "   AND cexpire>julianday('now')"
       "   AND length(cookie)>30",
-      uid, zRemoteAddr);
+      uid);
   if( zHash==0 ) zHash = db_text(0, "SELECT hex(randomblob(25))");
   zCookie = login_gen_user_cookie_value(zUsername, zHash);
-  cgi_set_cookie(zCookieName, zCookie, login_cookie_path(), expires);
+  cgi_set_cookie(zCookieName, zCookie, login_cookie_path(),
+                 bSessionCookie ? 0 : expires);
   record_login_attempt(zUsername, zIpAddr, 1);
-  db_multi_exec(
-                "UPDATE user SET cookie=%Q, ipaddr=%Q, "
+  db_multi_exec("UPDATE user SET cookie=%Q,"
                 "  cexpire=julianday('now')+%d/86400.0 WHERE uid=%d",
-                zHash, zRemoteAddr, expires, uid
-                );
-  free(zRemoteAddr);
-  free(zHash);
+                zHash, expires, uid);
+  fossil_free(zHash);
   if( zDest ){
     *zDest = zCookie;
   }else{
@@ -329,40 +310,34 @@ void login_set_user_cookie(
 **
 **    HASH/TIME/anonymous
 **
-** Where HASH is the sha1sum of TIME/IPADDR/SECRET, in which IPADDR
-** is the abbreviated IP address and SECRET is captcha-secret.
-**
-** If either zIpAddr or zRemoteAddr are NULL then REMOTE_ADDR
-** is used.
+** Where HASH is the sha1sum of TIME/SECRET, in which SECRET is captcha-secret.
 **
 ** If zCookieDest is not NULL then the generated cookie is assigned to
 ** *zCookieDest and the caller must eventually free() it.
+**
+** If bSessionCookie is true, the cookie will be a session cookie.
 */
-void login_set_anon_cookie(const char *zIpAddr, char **zCookieDest ){
+void login_set_anon_cookie(const char *zIpAddr, char **zCookieDest,
+                           int bSessionCookie ){
   const char *zNow;            /* Current time (julian day number) */
   char *zCookie;               /* The login cookie */
   const char *zCookieName;     /* Name of the login cookie */
   Blob b;                      /* Blob used during cookie construction */
-  char *zRemoteAddr;           /* Abbreviated IP address */
-  if(!zIpAddr){
-    zIpAddr = PD("REMOTE_ADDR","nil");
-  }
-  zRemoteAddr = ipPrefix(zIpAddr);
+  int expires = bSessionCookie ? 0 : 6*3600;
   zCookieName = login_cookie_name();
   zNow = db_text("0", "SELECT julianday('now')");
-  assert( zCookieName && zRemoteAddr && zIpAddr && zNow );
+  assert( zCookieName && zNow );
   blob_init(&b, zNow, -1);
-  blob_appendf(&b, "/%s/%s", zRemoteAddr, db_get("captcha-secret",""));
+  blob_appendf(&b, "/%s", db_get("captcha-secret",""));
   sha1sum_blob(&b, &b);
   zCookie = mprintf("%s/%s/anonymous", blob_buffer(&b), zNow);
   blob_reset(&b);
-  cgi_set_cookie(zCookieName, zCookie, login_cookie_path(), 6*3600);
+  cgi_set_cookie(zCookieName, zCookie, login_cookie_path(), expires);
   if( zCookieDest ){
     *zCookieDest = zCookie;
   }else{
     free(zCookie);
   }
-
 }
 
 /*
@@ -440,6 +415,7 @@ static int isHuman(const char *zAgent){
       return 1; /* IE11+ */
     }
     if( sqlite3_strglob("*AppleWebKit/[1-9]*(KHTML*", zAgent)==0 ) return 1;
+    if( sqlite3_strglob("*PaleMoon/[1-9]*", zAgent)==0 ) return 1;
     return 0;
   }
   if( strncmp(zAgent, "Opera/", 6)==0 ) return 1;
@@ -516,7 +492,7 @@ int login_self_register_available(const char *zNeeded){
   int rc;
   if( !db_get_boolean("self-register",0) ) return 0;
   if( zNeeded==0 ) return 1;
-  pCap = capability_add(0, db_get("default-perms", 0));
+  pCap = capability_add(0, db_get("default-perms", "u"));
   capability_expand(pCap);
   rc = capability_has_any(pCap, zNeeded);
   capability_free(pCap);
@@ -550,7 +526,10 @@ void login_page(void){
   char *zSha1Pw;
   const char *zIpAddr;         /* IP address of requestor */
   const char *zReferer;
-  int noAnon = P("noanon")!=0;
+  const int noAnon = P("noanon")!=0;
+  int rememberMe;              /* If true, use persistent cookie, else
+                                  session cookie. Toggled per
+                                  checkbox. */
 
   login_check_credentials();
   fossil_redirect_to_https_if_needed(1);
@@ -559,7 +538,6 @@ void login_page(void){
   zUsername = P("u");
   zPasswd = P("p");
   anonFlag = g.zLogin==0 && PB("anon");
-
   /* Handle log-out requests */
   if( P("out") ){
     login_clear_login_data();
@@ -635,8 +613,14 @@ void login_page(void){
   zIpAddr = PD("REMOTE_ADDR","nil");   /* Complete IP address for logging */
   zReferer = P("HTTP_REFERER");
   uid = login_is_valid_anonymous(zUsername, zPasswd, P("cs"));
+  if(zUsername==0){
+    /* Initial login page hit. */
+    rememberMe = 0;
+  }else{
+    rememberMe = P("remember")!=0;
+  }
   if( uid>0 ){
-    login_set_anon_cookie(zIpAddr, NULL);
+    login_set_anon_cookie(zIpAddr, NULL, rememberMe?0:1);
     record_login_attempt("anonymous", zIpAddr, 1);
     redirect_to_g();
   }
@@ -660,7 +644,7 @@ void login_page(void){
       ** where HASH is a random hex number, PROJECT is either project
       ** code prefix, and LOGIN is the user name.
       */
-      login_set_user_cookie(zUsername, uid, NULL);
+      login_set_user_cookie(zUsername, uid, NULL, rememberMe?0:1);
       redirect_to_g();
     }
   }
@@ -681,6 +665,7 @@ void login_page(void){
     }else{
       @ <p>Login as a named user to access page <b>%h(zAbbrev)</b>.
     }
+    fossil_free(zAbbrev);
   }
   if( g.sslNotAvailable==0
    && strncmp(g.zBaseURL,"https:",6)!=0
@@ -703,40 +688,49 @@ void login_page(void){
     @ <input type="submit" name="out" value="Logout"></p>
     @ </form>
   }else{
-    @ <table class="login_out">
-    @ <tr>
-    @   <td class="form_label">User ID:</td>
-    if( anonFlag ){
-      @ <td><input type="text" id="u" name="u" value="anonymous" size="30"></td>
+    unsigned int uSeed = captcha_seed();
+    if( g.zLogin==0 && (anonFlag || zGoto==0) ){
+      zAnonPw = db_text(0, "SELECT pw FROM user"
+                           " WHERE login='anonymous'"
+                           "   AND cap!=''");
     }else{
-      @ <td><input type="text" id="u" name="u" value="" size="30" /></td>
+      zAnonPw = 0;
     }
-    @ </tr>
-    @ <tr>
-    @  <td class="form_label">Password:</td>
-    @  <td><input type="password" id="p" name="p" value="" size="30" /></td>
-    @ </tr>
+    @ <table class="login_out">
     if( P("HTTPS")==0 ){
       @ <tr><td class="form_label">Warning:</td>
       @ <td><span class='securityWarning'>
-      @ Your password will be sent in the clear over an
-      @ unencrypted connection.
-      if( g.sslNotAvailable ){
-        @ No encrypted connection is available on this server.
-      }else{
+      @ Login information, including the password, 
+      @ will be sent in the clear over an unencrypted connection.
+      if( !g.sslNotAvailable ){
         @ Consider logging in at
         @ <a href='%s(g.zHttpsURL)'>%h(g.zHttpsURL)</a> instead.
       }
       @ </span></td></tr>
     }
-    if( g.zLogin==0 && (anonFlag || zGoto==0) ){
-      zAnonPw = db_text(0, "SELECT pw FROM user"
-                           " WHERE login='anonymous'"
-                           "   AND cap!=''");
+    @ <tr>
+    @   <td class="form_label" id="userlabel1">User ID:</td>
+    @   <td><input type="text" id="u" aria-labelledby="userlabel1" name="u" \
+    @ size="30" value="%s(anonFlag?"anonymous":"")"></td>
+    @ </tr>
+    @ <tr>
+    @  <td class="form_label" id="pswdlabel">Password:</td>
+    @  <td><input aria-labelledby="pswdlabel" type="password" id="p" \
+    @ name="p" value="" size="30" />\
+    if( zAnonPw && !noAnon ){
+      captcha_speakit_button(uSeed, "Speak password for \"anonymous\"");
     }
+    @ </td>
+    @ </tr>
     @ <tr>
     @   <td></td>
-    @   <td><input type="submit" name="in" value="Login"></td>
+    @   <td><input type="checkbox" name="remember" value="1" \
+    @ id="remember-me" %s(rememberMe ? "checked=\"checked\"" : "")>
+    @   <label for="remember-me">Remember me?</label></td>
+    @ </tr>
+    @ <tr>
+    @   <td></td>
+    @   <td><input type="submit" name="in" value="Login">
     @ </tr>
     if( !noAnon && login_self_register_available(0) ){
       @ <tr>
@@ -746,7 +740,6 @@ void login_page(void){
     }
     @ </table>
     if( zAnonPw && !noAnon ){
-      unsigned int uSeed = captcha_seed();
       const char *zDecoded = captcha_decode(uSeed);
       int bAutoCaptcha = db_get_boolean("auto-captcha", 0);
       char *zCaptcha = captcha_render(zDecoded);
@@ -761,7 +754,7 @@ void login_page(void){
       if( bAutoCaptcha ) {
          @ <input type="button" value="Fill out captcha" id='autofillButton' \
          @ data-af='%s(zDecoded)' />
-         style_load_one_js_file("login.js");
+         builtin_request_js("login.js");
       }
       @ </div>
       free(zCaptcha);
@@ -779,12 +772,15 @@ void login_page(void){
       @ <p>Change Password for user <b>%h(g.zLogin)</b>:</p>
       form_begin(0, "%R/login");
       @ <table>
-      @ <tr><td class="form_label">Old Password:</td>
-      @ <td><input type="password" name="p" size="30" /></td></tr>
-      @ <tr><td class="form_label">New Password:</td>
-      @ <td><input type="password" name="n1" size="30" /></td></tr>
-      @ <tr><td class="form_label">Repeat New Password:</td>
-      @ <td><input type="password" name="n2" size="30" /></td></tr>
+      @ <tr><td class="form_label" id="oldpw">Old Password:</td>
+      @ <td><input aria-labelledby="oldpw" type="password" name="p" \
+      @ size="30"/></td></tr>
+      @ <tr><td class="form_label" id="newpw">New Password:</td>
+      @ <td><input aria-labelledby="newpw" type="password" name="n1" \
+      @ size="30" /></td></tr>
+      @ <tr><td class="form_label" id="reppw">Repeat New Password:</td>
+      @ <td><input aria-labledby="reppw" type="password" name="n2" \
+      @ size="30" /></td></tr>
       @ <tr><td></td>
       @ <td><input type="submit" value="Change Password" /></td></tr>
       @ </table>
@@ -804,8 +800,7 @@ void login_page(void){
 static int login_transfer_credentials(
   const char *zLogin,          /* Login we are looking for */
   const char *zCode,           /* Project code of peer repository */
-  const char *zHash,           /* HASH from login cookie HASH/CODE/LOGIN */
-  const char *zRemoteAddr      /* Request comes from here */
+  const char *zHash            /* HASH from login cookie HASH/CODE/LOGIN */
 ){
   sqlite3 *pOther = 0;         /* The other repository */
   sqlite3_stmt *pStmt;         /* Query against the other repository */
@@ -833,20 +828,19 @@ static int login_transfer_credentials(
     zSQL = mprintf(
       "SELECT cexpire FROM user"
       " WHERE login=%Q"
-      "   AND ipaddr=%Q"
       "   AND length(cap)>0"
       "   AND length(pw)>0"
       "   AND cexpire>julianday('now')"
       "   AND constant_time_cmp(cookie,%Q)=0",
-      zLogin, zRemoteAddr, zHash
+      zLogin, zHash
     );
     pStmt = 0;
     rc = sqlite3_prepare_v2(pOther, zSQL, -1, &pStmt, 0);
     if( rc==SQLITE_OK && sqlite3_step(pStmt)==SQLITE_ROW ){
       db_multi_exec(
-        "UPDATE user SET cookie=%Q, ipaddr=%Q, cexpire=%.17g"
+        "UPDATE user SET cookie=%Q, cexpire=%.17g"
         " WHERE login=%Q",
-        zHash, zRemoteAddr,
+        zHash, 
         sqlite3_column_double(pStmt, 0), zLogin
       );
       nXfer++;
@@ -870,29 +864,26 @@ int login_is_special(const char *zLogin){
 }
 
 /*
-** Lookup the uid for a non-built-in user with zLogin and zCookie and
-** zRemoteAddr.  Return 0 if not found.
+** Lookup the uid for a non-built-in user with zLogin and zCookie.
+** Return 0 if not found.
 **
 ** Note that this only searches for logged-in entries with matching
-** zCookie (db: user.cookie) and zRemoteAddr (db: user.ipaddr)
-** entries.
+** zCookie (db: user.cookie) entries.
 */
 static int login_find_user(
   const char *zLogin,            /* User name */
-  const char *zCookie,           /* Login cookie value */
-  const char *zRemoteAddr        /* Abbreviated IP address for valid login */
+  const char *zCookie            /* Login cookie value */
 ){
   int uid;
   if( login_is_special(zLogin) ) return 0;
   uid = db_int(0,
     "SELECT uid FROM user"
     " WHERE login=%Q"
-    "   AND ipaddr=%Q"
     "   AND cexpire>julianday('now')"
     "   AND length(cap)>0"
     "   AND length(pw)>0"
     "   AND constant_time_cmp(cookie,%Q)=0",
-    zLogin, zRemoteAddr, zCookie
+    zLogin, zCookie
   );
   return uid;
 }
@@ -965,7 +956,6 @@ void login_check_credentials(void){
   int uid = 0;                  /* User id */
   const char *zCookie;          /* Text of the login cookie */
   const char *zIpAddr;          /* Raw IP address of the requestor */
-  char *zRemoteAddr;            /* Abbreviated IP address of the requestor */
   const char *zCap = 0;         /* Capability string */
   const char *zPublicPages = 0; /* GLOB patterns of public pages */
   const char *zLogin = 0;       /* Login user for credentials */
@@ -983,7 +973,7 @@ void login_check_credentials(void){
   ** This feature allows the "fossil ui" command to give the user
   ** full access rights without having to log in.
   */
-  zRemoteAddr = ipPrefix(zIpAddr = PD("REMOTE_ADDR","nil"));
+  zIpAddr = PD("REMOTE_ADDR","nil");
   if( ( cgi_is_loopback(zIpAddr)
        || (g.fSshClient & CGI_SSH_CLIENT)!=0 )
    && g.useLocalauth
@@ -1032,8 +1022,7 @@ void login_check_credentials(void){
       double rTime = atof(zArg);
       Blob b;
       blob_zero(&b);
-      blob_appendf(&b, "%s/%s/%s",
-                   zArg, zRemoteAddr, db_get("captcha-secret",""));
+      blob_appendf(&b, "%s/%s", zArg, db_get("captcha-secret",""));
       sha1sum_blob(&b, &b);
       if( fossil_strcmp(zHash, blob_str(&b))==0 ){
         uid = db_int(0,
@@ -1050,9 +1039,9 @@ void login_check_credentials(void){
       ** local user table, then the user table for project CODE if we
       ** are part of a login-group.
       */
-      uid = login_find_user(zUser, zHash, zRemoteAddr);
-      if( uid==0 && login_transfer_credentials(zUser,zArg,zHash,zRemoteAddr) ){
-        uid = login_find_user(zUser, zHash, zRemoteAddr);
+      uid = login_find_user(zUser, zHash);
+      if( uid==0 && login_transfer_credentials(zUser,zArg,zHash) ){
+        uid = login_find_user(zUser, zHash);
         if( uid ) record_login_attempt(zUser, zIpAddr, 1);
       }
     }
@@ -1164,7 +1153,7 @@ void login_check_credentials(void){
     const char *zUri = PD("REQUEST_URI","");
     zUri += (int)strlen(g.zTop);
     if( glob_match(pGlob, zUri) ){
-      login_set_capabilities(db_get("default-perms", 0), 0);
+      login_set_capabilities(db_get("default-perms", "u"), 0);
     }
     glob_free(pGlob);
   }
@@ -1228,7 +1217,7 @@ void login_set_capabilities(const char *zCap, unsigned flags){
                              p->ApndWiki = p->Hyperlink = p->Clone =
                              p->NewTkt = p->Password = p->RdAddr =
                              p->TktFmt = p->Attach = p->ApndTkt =
-                             p->ModWiki = p->ModTkt = p->Delete =
+                             p->ModWiki = p->ModTkt =
                              p->RdForum = p->WrForum = p->ModForum =
                              p->WrTForum = p->AdminForum =
                              p->EmailAlert = p->Announce = p->Debug = 1;
@@ -1237,7 +1226,6 @@ void login_set_capabilities(const char *zCap, unsigned flags){
       case 'o':   p->Read = 1;                                 break;
       case 'z':   p->Zip = 1;                                  break;
 
-      case 'd':   p->Delete = 1;                               break;
       case 'h':   p->Hyperlink = 1;                            break;
       case 'g':   p->Clone = 1;                                break;
       case 'p':   p->Password = 1;                             break;
@@ -1321,7 +1309,7 @@ int login_has_capability(const char *zCap, int nCap, u32 flgs){
       case 'a':  rc = p->Admin;     break;
       case 'b':  rc = p->Attach;    break;
       case 'c':  rc = p->ApndTkt;   break;
-      case 'd':  rc = p->Delete;    break;
+      /* d unused: see comment in capabilities.c */
       case 'e':  rc = p->RdAddr;    break;
       case 'f':  rc = p->NewWiki;   break;
       case 'g':  rc = p->Clone;     break;
@@ -1482,6 +1470,49 @@ void login_verify_csrf_secret(void){
   fossil_fatal("Cross-site request forgery attempt");
 }
 
+/*
+** Check to see if the candidate username zUserID is already used.
+** Return 1 if it is already in use.  Return 0 if the name is 
+** available for a self-registeration.
+*/
+static int login_self_choosen_userid_already_exists(const char *zUserID){
+  int rc = db_exists(
+    "SELECT 1 FROM user WHERE login=%Q "
+    "UNION ALL "
+    "SELECT 1 FROM event WHERE user=%Q OR euser=%Q",
+    zUserID, zUserID, zUserID
+  );
+  return rc;
+}
+
+/*
+** Check an email address and confirm that it is valid for self-registration.
+** The email address is known already to be well-formed.  Return true
+** if the email address is on the allowed list.
+**
+** The default behavior is that any valid email address is accepted.
+** But if the "auth-sub-email" setting exists and is not empty, then
+** it is a comma-separated list of GLOB patterns for email addresses
+** that are authorized to self-register.
+*/
+int authorized_subscription_email(const char *zEAddr){
+  char *zGlob = db_get("auth-sub-email",0);
+  Glob *pGlob;
+  char *zAddr;
+  int rc;
+
+  if( zGlob==0 || zGlob[0]==0 ) return 1;
+  zGlob = fossil_strtolwr(fossil_strdup(zGlob));
+  pGlob = glob_create(zGlob);
+  fossil_free(zGlob);
+
+  zAddr = fossil_strtolwr(fossil_strdup(zEAddr));
+  rc = glob_match(pGlob, zAddr);
+  fossil_free(zAddr);
+  glob_free(pGlob);
+  return rc!=0;
+}
+
 /*
 ** WEBPAGE: register
 **
@@ -1493,9 +1524,10 @@ void register_page(void){
   const char *zDName;
   unsigned int uSeed;
   const char *zDecoded;
-  char *zCaptcha;
   int iErrLine = -1;
   const char *zErr = 0;
+  int captchaIsCorrect = 0; /* True on a correct captcha */
+  char *zCaptcha = "";      /* Value of the captcha text */
   char *zPerms;             /* Permissions for the default user */
   int canDoAlerts = 0;      /* True if receiving email alerts is possible */
   int doAlerts = 0;         /* True if subscription is wanted too */
@@ -1506,7 +1538,7 @@ void register_page(void){
     style_footer();
     return;
   }
-  zPerms = db_get("default-perms", 0);
+  zPerms = db_get("default-perms", "u");
 
   /* Prompt the user for email alerts if this repository is configured for
   ** email alerts and if the default permissions include "7" */
@@ -1525,12 +1557,12 @@ void register_page(void){
   if( P("new")==0 || !cgi_csrf_safe(1) ){
     /* This is not a valid form submission.  Fall through into
     ** the form display */
-  }else if( !captcha_is_correct(1) ){
+  }else if( (captchaIsCorrect = captcha_is_correct(1))==0 ){
     iErrLine = 6;
     zErr = "Incorrect CAPTCHA";
-  }else if( strlen(zUserID)<3 ){
+  }else if( strlen(zUserID)<6 ){
     iErrLine = 1;
-    zErr = "User ID too short. Must be at least 3 characters.";
+    zErr = "User ID too short. Must be at least 6 characters.";
   }else if( sqlite3_strglob("*[^-a-zA-Z0-9_.]*",zUserID)==0 ){
     iErrLine = 1;
     zErr = "User ID may not contain spaces or special characters.";
@@ -1540,16 +1572,19 @@ void register_page(void){
   }else if( zEAddr[0]==0 ){
     iErrLine = 3;
     zErr = "Required";
-  }else if( email_copy_addr(zEAddr,0)==0 ){
+  }else if( email_address_is_valid(zEAddr,0)==0 ){
     iErrLine = 3;
     zErr = "Not a valid email address";
+  }else if( authorized_subscription_email(zEAddr)==0 ){
+    iErrLine = 3;
+    zErr = "Not an authorized email address";
   }else if( strlen(zPasswd)<6 ){
     iErrLine = 4;
     zErr = "Password must be at least 6 characters long";
   }else if( fossil_strcmp(zPasswd,zConfirm)!=0 ){
     iErrLine = 5;
     zErr = "Passwords do not match";
-  }else if( db_exists("SELECT 1 FROM user WHERE login=%Q", zUserID) ){
+  }else if( login_self_choosen_userid_already_exists(zUserID) ){
     iErrLine = 1;
     zErr = "This User ID is already taken. Choose something different.";
   }else if(
@@ -1558,9 +1593,10 @@ void register_page(void){
     ||
       /* Or if the email is a verify subscriber email with an associated
       ** user... */
-      db_exists(
-        "SELECT 1 FROM subscriber WHERE semail=%Q AND suname IS NOT NULL"
-        " AND sverified",zEAddr)
+      (alert_tables_exist() &&
+       db_exists(
+         "SELECT 1 FROM subscriber WHERE semail=%Q AND suname IS NOT NULL"
+         " AND sverified",zEAddr))
    ){
     iErrLine = 3;
     zErr = "This email address is already claimed by another user";
@@ -1570,16 +1606,24 @@ void register_page(void){
     Blob sql;
     int uid;
     char *zPass = sha1_shared_secret(zPasswd, zUserID, 0);
+    const char *zStartPerms = zPerms;
+    if( db_get_boolean("selfreg-verify",0) ){
+      /* If email verification is required for self-registration, initalize
+      ** the new user capabilities to just "7" (Sign up for email).  The
+      ** full "default-perms" permissions will be added when they click
+      ** the verification link on the email they are sent. */
+      zStartPerms = "7";
+    }
     blob_init(&sql, 0, 0);
     blob_append_sql(&sql,
        "INSERT INTO user(login,pw,cap,info,mtime)\n"
        "VALUES(%Q,%Q,%Q,"
        "'%q <%q>\nself-register from ip %q on '||datetime('now'),now())",
-       zUserID, zPass, zPerms, zDName, zEAddr, g.zIpAddr);
+       zUserID, zPass, zStartPerms, zDName, zEAddr, g.zIpAddr);
     fossil_free(zPass);
     db_multi_exec("%s", blob_sql_text(&sql));
     uid = db_int(0, "SELECT uid FROM user WHERE login=%Q", zUserID);
-    login_set_user_cookie(zUserID, uid, NULL);
+    login_set_user_cookie(zUserID, uid, NULL, 0);
     if( doAlerts ){
       /* Also make the new user a subscriber. */
       Blob hdr, body;
@@ -1589,12 +1633,16 @@ void register_page(void){
       const char *zGoto = P("g");
       int nsub = 0;
       char ssub[20];
+      CapabilityString *pCap;
+      pCap = capability_add(0, zPerms);
+      capability_expand(pCap);
       ssub[nsub++] = 'a';
-      if( g.perm.Read )    ssub[nsub++] = 'c';
-      if( g.perm.RdForum ) ssub[nsub++] = 'f';
-      if( g.perm.RdTkt )   ssub[nsub++] = 't';
-      if( g.perm.RdWiki )  ssub[nsub++] = 'w';
+      if( capability_has_any(pCap,"o") ) ssub[nsub++] = 'c';
+      if( capability_has_any(pCap,"2") ) ssub[nsub++] = 'f';
+      if( capability_has_any(pCap,"r") ) ssub[nsub++] = 't';
+      if( capability_has_any(pCap,"j") ) ssub[nsub++] = 'w';
       ssub[nsub] = 0;
+      capability_free(pCap);
       /* Also add the user to the subscriber table. */
       db_multi_exec(
         "INSERT INTO subscriber(semail,suname,"
@@ -1638,8 +1686,7 @@ void register_page(void){
         @ </pre></blockquote>
       }else{
         @ <p>An email has been sent to "%h(zEAddr)". That email contains a
-        @ hyperlink that you must click on in order to activate your
-        @ subscription.</p>
+        @ hyperlink that you must click to activate your account.</p>
       }
       alert_sender_free(pSender);
       if( zGoto ){
@@ -1652,7 +1699,11 @@ void register_page(void){
   }
 
   /* Prepare the captcha. */
-  uSeed = captcha_seed();
+  if( captchaIsCorrect ){
+    uSeed = strtoul(P("captchaseed"),0,10);
+  }else{
+    uSeed = captcha_seed();
+  }
   zDecoded = captcha_decode(uSeed);
   zCaptcha = captcha_render(zDecoded);
 
@@ -1665,23 +1716,26 @@ void register_page(void){
   @ <p><input type="hidden" name="captchaseed" value="%u(uSeed)" />
   @ <table class="login_out">
   @ <tr>
-  @   <td class="form_label" align="right">User ID:</td>
-  @   <td><input type="text" name="u" value="%h(zUserID)" size="30"></td>
+  @   <td class="form_label" align="right" id="uid">User ID:</td>
+  @   <td><input aria-labelledby="uid" type="text" name="u" \
+  @ value="%h(zUserID)" size="30"></td>
   @
   if( iErrLine==1 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
   }
   @ <tr>
-  @   <td class="form_label" align="right">Display Name:</td>
-  @   <td><input type="text" name="dn" value="%h(zDName)" size="30"></td>
+  @   <td class="form_label" align="right" id="dpyname">Display Name:</td>
+  @   <td><input aria-labelledby="dpyname" type="text" name="dn" \
+  @ value="%h(zDName)" size="30"></td>
   @ </tr>
   if( iErrLine==2 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
   }
   @ </tr>
   @ <tr>
-  @   <td class="form_label" align="right">Email Address:</td>
-  @   <td><input type="text" name="ea" value="%h(zEAddr)" size="30"></td>
+  @   <td class="form_label" align="right" id="emaddr">Email Address:</td>
+  @   <td><input aria-labelledby="emaddr" type="text" name="ea" \
+  @ value="%h(zEAddr)" size="30"></td>
   @ </tr>
   if( iErrLine==3 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
@@ -1689,29 +1743,34 @@ void register_page(void){
   if( canDoAlerts ){
     int a = atoi(PD("alerts","1"));
     @ <tr>
-    @   <td class="form_label" align="right">Email&nbsp;Alerts?</td>
-    @   <td><select size='1' name='alerts'>
+    @   <td class="form_label" align="right" id="emalrt">Email&nbsp;Alerts?</td>
+    @   <td><select aria-labelledby="emalrt" size='1' name='alerts'>
     @       <option value="1" %s(a?"selected":"")>Yes</option>
     @       <option value="0" %s(!a?"selected":"")>No</option>
     @   </select></td></tr>
   }
   @ <tr>
-  @   <td class="form_label" align="right">Password:</td>
-  @   <td><input type="password" name="p" value="%h(zPasswd)" size="30"></td>
+  @   <td class="form_label" align="right" id="pswd">Password:</td>
+  @   <td><input aria-labelledby="pswd" type="password" name="p" \
+  @ value="%h(zPasswd)" size="30"></td>
   @ <tr>
   if( iErrLine==4 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
   }
   @ <tr>
-  @   <td class="form_label" align="right">Confirm:</td>
-  @   <td><input type="password" name="cp" value="%h(zConfirm)" size="30"></td>
+  @   <td class="form_label" align="right" id="pwcfrm">Confirm:</td>
+  @   <td><input aria-labelledby="pwcfrm" type="password" name="cp" \
+  @ value="%h(zConfirm)" size="30"></td>
   @ </tr>
   if( iErrLine==5 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
   }
   @ <tr>
-  @   <td class="form_label" align="right">Captcha:</td>
-  @   <td><input type="text" name="captcha" value="" size="30"></td>
+  @   <td class="form_label" align="right" id="cptcha">Captcha:</td>
+  @   <td><input type="text" name="captcha" aria-labelledby="cptcha" \
+  @ value="%h(captchaIsCorrect?zDecoded:"")" size="30">
+  captcha_speakit_button(uSeed, "Speak the captcha text");
+  @   </td>
   @ </tr>
   if( iErrLine==6 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
