@@ -70,7 +70,7 @@ struct Manifest {
   Blob content;         /* The original content blob */
   int type;             /* Type of artifact.  One of CFTYPE_xxxxx */
   int rid;              /* The blob-id for this manifest */
-  char *zBaseline;      /* Baseline manifest.  The B card. */
+  const char *zBaseline;/* Baseline manifest.  The B card. */
   Manifest *pBaseline;  /* The actual baseline manifest */
   char *zComment;       /* Decoded comment.  The C card. */
   double rDate;         /* Date and time from D card.  0.0 if no D card. */
@@ -132,7 +132,7 @@ static struct {
                      ** but we must limit for historical compatibility */
   /* CFTYPE_CLUSTER    2 */ { "MZ",            "MZ"          },
   /* CFTYPE_CONTROL    3 */ { "DTUZ",          "DTUZ"        },
-  /* CFTYPE_WIKI       4 */ { "DLNPUWZ",       "DLUWZ"       },
+  /* CFTYPE_WIKI       4 */ { "CDLNPUWZ",      "DLUWZ"       },
   /* CFTYPE_TICKET     5 */ { "DJKUZ",         "DJKUZ"       },
   /* CFTYPE_ATTACHMENT 6 */ { "ACDNUZ",        "ADZ"         },
   /* CFTYPE_EVENT      7 */ { "CDENPTUWZ",     "DEWZ"        },
@@ -320,14 +320,17 @@ static void remove_pgp_signature(char **pz, int *pn){
 **   0123456789 123456789 123456789 123456789
 **   Z aea84f4f863865a8d59d0384e4d2a41c
 */
-static int verify_z_card(const char *z, int n){
+static int verify_z_card(const char *z, int n, Blob *pErr){
+  const char *zHash;
   if( n<35 ) return 0;
   if( z[n-35]!='Z' || z[n-34]!=' ' ) return 0;
   md5sum_init();
   md5sum_step_text(z, n-35);
-  if( memcmp(&z[n-33], md5sum_finish(0), 32)==0 ){
+  zHash = md5sum_finish(0);
+  if( memcmp(&z[n-33], zHash, 32)==0 ){
     return 1;
   }else{
+    blob_appendf(pErr, "incorrect Z-card cksum: expected %.32s", zHash);
     return 2;
   }
 }
@@ -384,6 +387,19 @@ static char next_card(ManifestText *p){
 */
 #define SYNTAX(T)  {zErr=(T); goto manifest_syntax_error;}
 
+/*
+** A cache of manifest IDs which manifest_parse() has seen in this
+** session.
+*/
+static Bag seenManifests =  Bag_INIT;
+/*
+** Frees all memory owned by the manifest "has-seen" cache.  Intended
+** to be called only from the app's atexit() handler.
+*/
+void manifest_clear_cache(){
+  bag_clear(&seenManifests);
+}
+
 /*
 ** Parse a blob into a Manifest object.  The Manifest object
 ** takes over the input blob and will free it when the
@@ -427,7 +443,6 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   int isRepeat;
   int nSelfTag = 0;     /* Number of T cards referring to this manifest */
   int nSimpleTag = 0;   /* Number of T cards with "+" prefix */
-  static Bag seen;
   const char *zErr = 0;
   unsigned int m;
   unsigned int seenCard = 0;   /* Which card types have been seen */
@@ -435,11 +450,11 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
   if( rid==0 ){
     isRepeat = 1;
-  }else if( bag_find(&seen, rid) ){
+  }else if( bag_find(&seenManifests, rid) ){
     isRepeat = 1;
   }else{
     isRepeat = 0;
-    bag_insert(&seen, rid);
+    bag_insert(&seenManifests, rid);
   }
 
   /* Every structural artifact ends with a '\n' character.  Exit early
@@ -468,11 +483,19 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   }
   /* Then verify the Z-card.
   */
-  if( verify_z_card(z, n)==2 ){
+#if 1
+  /* Disable this ***ONLY*** (ONLY!) when testing hand-written inputs
+     for card-related syntax errors. */
+  if( verify_z_card(z, n, pErr)==2 ){
     blob_reset(pContent);
-    blob_appendf(pErr, "incorrect Z-card cksum");
     return 0;
   }
+#else
+#warning ACHTUNG - z-card check is disabled for testing purposes.
+  if(0 && verify_z_card(NULL, 0, NULL)){
+    /*avoid unused static func error*/
+  }
+#endif
 
   /* Allocate a Manifest object to hold the parsed control artifact.
   */
@@ -589,6 +612,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         p->rEventDate = db_double(0.0,"SELECT julianday(%Q)", next_token(&x,0));
         if( p->rEventDate<=0.0 ) SYNTAX("malformed date on E-card");
         p->zEventId = next_token(&x, &sz);
+        if( p->zEventId==0 ) SYNTAX("missing hash on E-card");
         if( !hname_validate(p->zEventId, sz) ){
           SYNTAX("malformed hash on E-card");
         }
@@ -613,6 +637,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         }
         zUuid = next_token(&x, &sz);
         if( p->zBaseline==0 || zUuid!=0 ){
+          if( zUuid==0 ) SYNTAX("missing hash on F-card");
           if( !hname_validate(zUuid,sz) ){
             SYNTAX("F-card hash invalid");
           }
@@ -631,13 +656,20 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
                                     p->nFileAlloc*sizeof(p->aFile[0]) );
         }
         i = p->nFile++;
+        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
+          SYNTAX("incorrect F-card sort order");
+        }
+        if( file_is_reserved_name(zName,-1) ){
+          /* If reserved names leaked into historical manifests due to
+          ** slack oversight by older versions of Fossil, simply ignore
+          ** those files */
+          p->nFile--;
+          break;
+        }
         p->aFile[i].zName = zName;
         p->aFile[i].zUuid = zUuid;
         p->aFile[i].zPerm = zPerm;
         p->aFile[i].zPrior = zPriorName;
-        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
-          SYNTAX("incorrect F-card sort order");
-        }
         p->type = CFTYPE_MANIFEST;
         break;
       }
@@ -1052,7 +1084,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
 manifest_syntax_error:
   {
-    char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
+    char *zUuid = rid_to_uuid(rid);
     if( zUuid ){
       blob_appendf(pErr, "artifact [%s] ", zUuid);
       fossil_free(zUuid);
@@ -1143,6 +1175,7 @@ void manifest_test_parse_cmd(void){
     blob_reset(&err);
     manifest_destroy(p);
   }
+  blob_reset(&b);
 }
 
 /*
@@ -1314,7 +1347,7 @@ static int filename_to_fnid(const char *zFilename){
 ** Compute an appropriate mlink.mperm integer for the permission string
 ** of a file.
 */
-int manifest_file_mperm(ManifestFile *pFile){
+int manifest_file_mperm(const ManifestFile *pFile){
   int mperm = PERM_REG;
   if( pFile && pFile->zPerm){
     if( strstr(pFile->zPerm,"x")!=0 ){
@@ -1703,7 +1736,7 @@ static int manifest_add_checkin_linkages(
   int rid,                   /* The RID of the check-in */
   Manifest *p,               /* Manifest for this check-in */
   int nParent,               /* Number of parents for this check-in */
-  char **azParent            /* hashes for each parent */
+  char * const * azParent    /* hashes for each parent */
 ){
   int i;
   int parentid = 0;
@@ -1815,7 +1848,7 @@ void manifest_crosslink_begin(void){
   manifest_crosslink_busy = 1;
   db_begin_transaction();
   db_multi_exec(
-     "CREATE TEMP TABLE pending_tkt(uuid TEXT UNIQUE);"
+     "CREATE TEMP TABLE pending_xlink(id TEXT PRIMARY KEY)WITHOUT ROWID;"
      "CREATE TEMP TABLE time_fudge("
      "  mid INTEGER PRIMARY KEY,"    /* The rid of a manifest */
      "  m1 REAL,"                    /* The timestamp on mid */
@@ -1825,6 +1858,17 @@ void manifest_crosslink_begin(void){
   );
 }
 
+/*
+** Add a new entry to the pending_xlink table.
+*/
+static void add_pending_crosslink(char cType, const char *zId){
+  assert( manifest_crosslink_busy==1 );
+  db_multi_exec(
+    "INSERT OR IGNORE INTO pending_xlink VALUES('%c%q')",
+    cType, zId
+  );
+}
+
 #if INTERFACE
 /* Timestamps might be adjusted slightly to ensure that check-ins appear
 ** on the timeline in chronological order.  This is the maximum amount
@@ -1866,16 +1910,24 @@ int manifest_crosslink_end(int flags){
     manifest_reparent_checkin(rid, zValue);
   }
   db_finalize(&q);
-  db_prepare(&q, "SELECT uuid FROM pending_tkt");
+  db_prepare(&q, "SELECT id FROM pending_xlink");
   while( db_step(&q)==SQLITE_ROW ){
-    const char *zUuid = db_column_text(&q, 0);
-    ticket_rebuild_entry(zUuid);
-    if( permitHooks && rc==TH_OK ){
-      rc = xfer_run_script(zScript, zUuid, 0);
+    const char *zId = db_column_text(&q, 0);
+    char cType;
+    if( zId==0 || zId[0]==0 ) continue;
+    cType = zId[0];
+    zId++;
+    if( cType=='t' ){
+      ticket_rebuild_entry(zId);
+      if( permitHooks && rc==TH_OK ){
+        rc = xfer_run_script(zScript, zId, 0);
+      }
+    }else if( cType=='w' ){
+      backlink_wiki_refresh(zId);
     }
   }
   db_finalize(&q);
-  db_multi_exec("DROP TABLE pending_tkt");
+  db_multi_exec("DROP TABLE pending_xlink");
 
   /* If multiple check-ins happen close together in time, adjust their
   ** times by a few milliseconds to make sure they appear in chronological
@@ -2046,7 +2098,7 @@ static int tag_compare(const void *a, const void *b){
 
 /*
 ** Scan artifact rid/pContent to see if it is a control artifact of
-** any key:
+** any type:
 **
 **      *  Manifest
 **      *  Control
@@ -2055,6 +2107,7 @@ static int tag_compare(const void *a, const void *b){
 **      *  Cluster
 **      *  Attachment
 **      *  Event
+**      *  Forum post
 **
 ** If the input is a control artifact, then make appropriate entries
 ** in the auxiliary tables of the database in order to crosslink the
@@ -2086,8 +2139,9 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
   }else if( (p = manifest_parse(pContent, rid, 0))==0 ){
     assert( blob_is_reset(pContent) || pContent==0 );
     if( (flags & MC_NO_ERRORS)==0 ){
-      fossil_error(1, "syntax error in manifest [%S]",
-                   db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid));
+      char * zErrUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid);
+      fossil_error(1, "syntax error in manifest [%S]", zErrUuid);
+      fossil_free(zErrUuid);
     }
     return 0;
   }
@@ -2147,7 +2201,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       );
       zCom = db_text(0, "SELECT coalesce(ecomment, comment) FROM event"
                         " WHERE rowid=last_insert_rowid()");
-      wiki_extract_links(zCom, rid, 0, p->rDate, 1, WIKI_INLINE);
+      backlink_extract(zCom, 0, rid, BKLNK_COMMENT, p->rDate, 1);
       fossil_free(zCom);
 
       /* If this is a delta-manifest, record the fact that this repository
@@ -2196,6 +2250,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
           case '*':  type = 2;  break;  /* Propagate to descendants */
           default:
             fossil_error(1, "unknown tag type in manifest: %s", p->aTag);
+            manifest_destroy(p);
             return 0;
         }
         tag_insert(&p->aTag[i].zName[1], type, p->aTag[i].zValue,
@@ -2211,6 +2266,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     int tagid = tag_findid(zTag, 1);
     int prior;
     char *zComment;
+    const char *zPrefix;
     int nWiki;
     char zLength[40];
     while( fossil_isspace(p->zWiki[0]) ) p->zWiki++;
@@ -2227,12 +2283,40 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     if( prior ){
       content_deltify(prior, &rid, 1, 0);
     }
-    if( nWiki>0 ){
-      zComment = mprintf("Changes to wiki page [%h]", p->zWikiTitle);
+    if( nWiki<=0 ){
+      zPrefix = "Deleted";
+    }else if( !prior ){
+      zPrefix = "Added";
     }else{
-      zComment = mprintf("Deleted wiki page [%h]", p->zWikiTitle);
+      zPrefix = "Changes to";
+    }
+    switch( wiki_page_type(p->zWikiTitle) ){
+      case WIKITYPE_CHECKIN: {
+        zComment = mprintf("%s wiki for check-in [%S]", zPrefix,
+                           p->zWikiTitle+8);
+        break;
+      }
+      case WIKITYPE_BRANCH: {
+        zComment = mprintf("%s wiki for branch [/timeline?r=%t|%h]",
+                           zPrefix, p->zWikiTitle+7, p->zWikiTitle+7);
+        break;
+      }
+      case WIKITYPE_TAG: {
+        zComment = mprintf("%s wiki for tag [/timeline?t=%t|%h]",
+                           zPrefix, p->zWikiTitle+4, p->zWikiTitle+4);
+        break;
+      }
+      default: {
+        zComment = mprintf("%s wiki page [%h]", zPrefix, p->zWikiTitle);
+        break;
+      }
     }
     search_doc_touch('w',rid,p->zWikiTitle);
+    if( manifest_crosslink_busy ){
+      add_pending_crosslink('w',p->zWikiTitle);
+    }else{
+      backlink_wiki_refresh(p->zWikiTitle);
+    }
     db_multi_exec(
       "REPLACE INTO event(type,mtime,objid,user,comment,"
       "                  bgcolor,euser,ecomment)"
@@ -2333,8 +2417,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     zTag = mprintf("tkt-%s", p->zTicketUuid);
     tag_insert(zTag, 1, 0, rid, p->rDate, rid);
     fossil_free(zTag);
-    db_multi_exec("INSERT OR IGNORE INTO pending_tkt VALUES(%Q)",
-                  p->zTicketUuid);
+    add_pending_crosslink('t',p->zTicketUuid);
     /* Locate and update comment for any attachments */
     db_prepare(&qatt,
        "SELECT attachid, src, target, filename FROM attachment"
@@ -2371,7 +2454,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     ** process the attachment artifact before the artifact to
     ** which it is attached!) */
     char attachToType = 'w';
-    if( fossil_is_uuid(p->zAttachTarget) ){
+    if( fossil_is_artifact_hash(p->zAttachTarget) ){
       if( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
             p->zAttachTarget)
         ){
@@ -2543,6 +2626,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     char *zFType;
     char *zTitle;
     schema_forum();
+    search_doc_touch('f', rid, 0);
     froot = p->zThreadRoot ? uuid_to_rid(p->zThreadRoot, 1) : rid;
     fprev = p->nParent ? uuid_to_rid(p->azParent[0],1) : 0;
     firt = p->zInReplyTo ? uuid_to_rid(p->zInReplyTo,1) : 0;
