@@ -195,8 +195,16 @@ static int report_query_authorizer(
          "tag",
          "tagxref",
          "unversioned",
+         "backlink",
       };
       int i;
+      if( zArg1==0 ){
+        /* Some legacy versions of SQLite will sometimes send spurious
+        ** READ authorizations that have no table name.  These can be
+        ** ignored. */
+        rc = SQLITE_IGNORE;
+        break;
+      }
       if( fossil_strncmp(zArg1, "fx_", 3)==0 ){
         break;
       }
@@ -224,11 +232,11 @@ static int report_query_authorizer(
 ** Activate the query authorizer
 */
 void report_restrict_sql(char **pzErr){
-  sqlite3_set_authorizer(g.db, report_query_authorizer, (void*)pzErr);
+  db_set_authorizer(report_query_authorizer,(void*)pzErr,"Ticket-Report");
   sqlite3_limit(g.db, SQLITE_LIMIT_VDBE_OP, 10000);
 }
 void report_unrestrict_sql(void){
-  sqlite3_set_authorizer(g.db, 0, 0);
+  db_clear_authorizer();
 }
 
 
@@ -505,9 +513,9 @@ void view_edit(void){
 */
 static void report_format_hints(void){
   char *zSchema;
-  zSchema = db_text(0,"SELECT sql FROM sqlite_master WHERE name='ticket'");
+  zSchema = db_text(0,"SELECT sql FROM sqlite_schema WHERE name='ticket'");
   if( zSchema==0 ){
-    zSchema = db_text(0,"SELECT sql FROM repository.sqlite_master"
+    zSchema = db_text(0,"SELECT sql FROM repository.sqlite_schema"
                         " WHERE name='ticket'");
   }
   @ <hr /><h3>TICKET Schema</h3>
@@ -665,7 +673,7 @@ static int generate_html(
 ){
   struct GenerateHTML *pState = (struct GenerateHTML*)pUser;
   int i;
-  const char *zTid;  /* Ticket UUID.  (value of column named '#') */
+  const char *zTid;  /* Ticket hash.  (value of column named '#') */
   const char *zBg = 0; /* Use this background color */
 
   /* Do initialization
@@ -674,7 +682,7 @@ static int generate_html(
     /* Turn off the authorizer.  It is no longer doing anything since the
     ** query has already been prepared.
     */
-    sqlite3_set_authorizer(g.db, 0, 0);
+    db_clear_authorizer();
 
     /* Figure out the number of columns, the column that determines background
     ** color, and whether or not this row of data is represented by multiple
