@@ -198,17 +198,19 @@ void onoff_attribute(
     if( iQ!=iVal ){
       login_verify_csrf_secret();
       db_set(zVar, iQ ? "1" : "0", 0);
+      setup_incr_cfgcnt();
       admin_log("Set option [%q] to [%q].",
                 zVar, iQ ? "on" : "off");
       iVal = iQ;
     }
   }
-  @ <label><input type="checkbox" name="%s(zQParm)"
+  @ <label><input type="checkbox" name="%s(zQParm)" \
+  @ aria-label="%s(zLabel[0]?zLabel:zQParm)" \
   if( iVal ){
-    @ checked="checked"
+    @ checked="checked" \
   }
   if( disabled ){
-    @ disabled="disabled"
+    @ disabled="disabled" \
   }
   @ /> <b>%s(zLabel)</b></label>
 }
@@ -229,13 +231,14 @@ void entry_attribute(
   if( zQ && fossil_strcmp(zQ,zVal)!=0 ){
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
+    setup_incr_cfgcnt();
     db_set(zVar, zQ, 0);
     admin_log("Set entry_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     zVal = zQ;
   }
-  @ <input type="text" id="%s(zQParm)" name="%s(zQParm)" value="%h(zVal)" \
-  @ size="%d(width)" \
+  @ <input aria-label="%h(zLabel[0]?zLabel:zQParm)" type="text" \
+  @ id="%s(zQParm)" name="%s(zQParm)" value="%h(zVal)" size="%d(width)" \
   if( disabled ){
     @ disabled="disabled" \
   }
@@ -260,17 +263,19 @@ const char *textarea_attribute(
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
     db_set(zVar, zQ, 0);
+    setup_incr_cfgcnt();
     admin_log("Set textarea_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     z = zQ;
   }
   if( rows>0 && cols>0 ){
-    @ <textarea id="id%s(zQP)" name="%s(zQP)" rows="%d(rows)"
+    @ <textarea id="id%s(zQP)" name="%s(zQP)" rows="%d(rows)" \
+    @ aria-label="%h(zLabel[0]?zLabel:zQP)" \
     if( disabled ){
-      @ disabled="disabled"
+      @ disabled="disabled" \
     }
     @ cols="%d(cols)">%h(z)</textarea>
-    if( zLabel && *zLabel ){
+    if( *zLabel ){
       @ <span class="textareaLabel">%s(zLabel)</span>
     }
   }
@@ -295,11 +300,12 @@ void multiple_choice_attribute(
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
     db_set(zVar, zQ, 0);
+    setup_incr_cfgcnt();
     admin_log("Set multiple_choice_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     z = zQ;
   }
-  @ <select size="1" name="%s(zQP)" id="id%s(zQP)">
+  @ <select aria-label="%h(zLabel)" size="1" name="%s(zQP)" id="id%s(zQP)">
   for(i=0; i<nChoice*2; i+=2){
     const char *zSel = fossil_strcmp(azChoice[i],z)==0 ? " selected" : "";
     @ <option value="%h(azChoice[i])"%s(zSel)>%h(azChoice[i+1])</option>
@@ -378,6 +384,15 @@ void setup_access(void){
   @ </p>
   @
   @ <hr />
+  onoff_attribute("Enable /artifact_stats",
+     "artifact_stats_enable", "artifact_stats_enable", 0, 0);
+  @ <p>When enabled, the %h(g.zBaseURL)/artifact_stats URL is available to all
+  @ users.  When disabled (the default) only users with check-in privilege may
+  @ access the /artifact_stats page.
+  @ (Property: "artifact_stats_enable")
+  @ </p>
+  @
+  @ <hr />
   onoff_attribute("Allow REMOTE_USER authentication",
      "remote_user_ok", "remote_user_ok", 0, 0);
   @ <p>When enabled, if the REMOTE_USER environment variable is set to the
@@ -391,20 +406,11 @@ void setup_access(void){
      "http_authentication_ok", "http_authentication_ok", 0, 0);
   @ <p>When enabled, allow the use of the HTTP_AUTHENTICATION environment
   @ variable or the "Authentication:" HTTP header to find the username and
-  @ password. This is another way of supporting Basic Authenitication.
+  @ password. This is another way of supporting Basic Authentication.
   @ (Property: "http_authentication_ok")
   @ </p>
   @
   @ <hr />
-  entry_attribute("IP address terms used in login cookie", 3,
-                  "ip-prefix-terms", "ipt", "2", 0);
-  @ <p>The number of octets of of the IP address used in the login cookie.
-  @ Set to zero to omit the IP address from the login cookie.  A value of
-  @ 2 is recommended.
-  @ (Property: "ip-prefix-terms")
-  @ </p>
-  @
-  @ <hr />
   entry_attribute("Login expiration time", 6, "cookie-expire", "cex",
                   "8766", 0);
   @ <p>The number of hours for which a login is valid.  This must be a
@@ -490,8 +496,11 @@ void setup_access(void){
                   "pubpage", "", 0);
   @ <p>A comma-separated list of glob patterns for pages that are accessible
   @ without needing a login and using the privileges given by the
-  @ "Default privileges" setting below.  Example use case: Set this field
-  @ to "/doc/trunk/www/*" to give anonymous users read-only permission to the
+  @ "Default privileges" setting below. 
+  @
+  @ <p>Example use case: Set this field to "/doc/trunk/www/*" and set
+  @ the "Default privileges" to include the "o" privilege
+  @ to give anonymous users read-only permission to the
   @ latest version of the embedded documentation in the www/ folder without
   @ allowing them to see the rest of the source code.
   @ (Property: "public-pages")
@@ -499,13 +508,40 @@ void setup_access(void){
 
   @ <hr />
   onoff_attribute("Allow users to register themselves",
-                  "self-register", "selfregister", 0, 0);
-  @ <p>Allow users to register themselves through the HTTP UI.
-  @ The registration form always requires filling in a CAPTCHA
-  @ (<em>auto-captcha</em> setting is ignored). Still, bear in mind that anyone
-  @ can register under any user name. This option is useful for public projects
-  @ where you do not want everyone in any ticket discussion to be named
-  @ "Anonymous".  (Property: "self-register")</p>
+                  "self-register", "selfreg", 0, 0);
+  @ <p>Allow users to register themselves on the /register webpage.
+  @ A self-registration creates a new entry in the USER table and
+  @ perhaps also in the SUBSCRIBER table if email notification is
+  @ enabled.
+  @ (Property: "self-register")</p>
+
+  @ <hr />
+  onoff_attribute("Email verification required for self-registration",
+                  "selfreg-verify", "sfverify", 0, 0);
+  @ <p>If enabled, self-registration creates a new entry in the USER table
+  @ with only capabilities "7".  The default user capabilities are not
+  @ added until the email address associated with the self-registration
+  @ has been verified. This setting only makes sense if
+  @ email notifications are enabled.
+  @ (Property: "selfreg-verify")</p>
+
+  @ <hr />
+  onoff_attribute("Allow anonymous subscriptions",
+                  "anon-subscribe", "anonsub", 1, 0);
+  @ <p>If disabled, email notification subscriptions are only allowed
+  @ for users with a login.  If Nobody or Anonymous visit the /subscribe
+  @ page, they are redirected to /register or /login.
+  @ (Property: "anon-subscribe")</p>
+
+  @ <hr />
+  entry_attribute("Authorized subscription email addresses", 35,
+                  "auth-sub-email", "asemail", "", 0);
+  @ <p>This is a comma-separated list of GLOB patterns that specify
+  @ email addresses that are authorized to subscriptions.  If blank
+  @ (the usual case), then any email address can be used to self-register.
+  @ This setting is used to limit subscriptions to members of a particular
+  @ organization or group based on their email address.
+  @ (Property: "auth-sub-email")</p>
 
   @ <hr />
   entry_attribute("Default privileges", 10, "default-perms",
@@ -578,21 +614,26 @@ void setup_login_group(void){
     login_insert_csrf_secret();
     @ <blockquote><table border="0">
     @
-    @ <tr><th align="right">Repository filename in group to join:</th>
+    @ <tr><th align="right" id="rfigtj">Repository filename \
+    @ in group to join:</th>
     @ <td width="5"></td><td>
-    @ <input type="text" size="50" value="%h(zRepo)" name="repo"></td></tr>
+    @ <input aria-labelledby="rfigtj" type="text" size="50" \
+    @ value="%h(zRepo)" name="repo"></td></tr>
     @
-    @ <tr><th align="right">Login on the above repo:</th>
+    @ <tr><th align="right" id="lotar">Login on the above repo:</th>
     @ <td width="5"></td><td>
-    @ <input type="text" size="20" value="%h(zLogin)" name="login"></td></tr>
+    @ <input aria-labelledby="lotar" type="text" size="20" \
+    @ value="%h(zLogin)" name="login"></td></tr>
     @
-    @ <tr><th align="right">Password:</th>
+    @ <tr><th align="right" id="lgpw">Password:</th>
     @ <td width="5"></td><td>
-    @ <input type="password" size="20" name="pw"></td></tr>
+    @ <input aria-labelledby="lgpw" type="password" size="20" name="pw">\
+    @ </td></tr>
     @
-    @ <tr><th align="right">Name of login-group:</th>
+    @ <tr><th align="right" id="nolg">Name of login-group:</th>
     @ <td width="5"></td><td>
-    @ <input type="text" size="30" value="%h(zNewName)" name="newname">
+    @ <input aria-labelledby="nolg" type="text" size="30" \
+    @ value="%h(zNewName)" name="newname">
     @ (only used if creating a new login-group).</td></tr>
     @
     @ <tr><td colspan="3" align="center">
@@ -735,6 +776,13 @@ void setup_timeline(void){
     @ %s(zTmDiff) hours ahead of UTC.</p>
   }
   @ <p>(Property: "timeline-utc")
+
+  @ <hr />
+  multiple_choice_attribute("Style", "timeline-default-style",
+            "tdss", "0", N_TIMELINE_VIEW_STYLE, timeline_view_styles);
+  @ <p>The default timeline viewing style, for when the user has not
+  @ specified an alternative.  (Property: "timeline-default-style")</p>
+
   @ <hr />
   multiple_choice_attribute("Per-Item Time Format", "timeline-date-format",
             "tdf", "0", count(azTimeFormats)/2, azTimeFormats);
@@ -841,24 +889,29 @@ void setup_settings(void){
   }
   @ <br /><input type="submit"  name="submit" value="Apply Changes" />
   @ </td><td style="width:50px;"></td><td valign="top">
+  @ <table>
   for(i=0, pSet=aSetting; i<nSetting; i++, pSet++){
-    if( pSet->width!=0 && !pSet->forceTextArea ){
+    if( pSet->width>0 && !pSet->forceTextArea ){
       int hasVersionableValue = pSet->versionable &&
           (db_get_versioned(pSet->name, NULL)!=0);
+      @ <tr><td>
+      @ <a href='%R/help?cmd=%s(pSet->name)'>%h(pSet->name)</a>
+      if( pSet->versionable ){
+        @  (v)
+      } else {
+        @
+      }
+      @</td><td>
       entry_attribute("", /*pSet->width*/ 25, pSet->name,
                       pSet->var!=0 ? pSet->var : pSet->name,
                       (char*)pSet->def, hasVersionableValue);
-      @ <a href='%R/help?cmd=%s(pSet->name)'>%h(pSet->name)</a>
-      if( pSet->versionable ){
-        @  (v)<br />
-      } else {
-        @ <br />
-      }
+      @</td></tr>
     }
   }
+  @</table>
   @ </td><td style="width:50px;"></td><td valign="top">
   for(i=0, pSet=aSetting; i<nSetting; i++, pSet++){
-    if( pSet->width!=0 && pSet->forceTextArea ){
+    if( pSet->width>0 && pSet->forceTextArea ){
       int hasVersionableValue = db_get_versioned(pSet->name, NULL)!=0;
       @ <a href='%R/help?cmd=%s(pSet->name)'>%s(pSet->name)</a>
       if( pSet->versionable ){
@@ -1006,19 +1059,30 @@ void setup_wiki(void){
   @ </ul>
   @ (Property: "wiki-about")</p>
   @ <hr />
-  onoff_attribute("Enable WYSIWYG Wiki Editing",
-                  "wysiwyg-wiki", "wysiwyg-wiki", 0, 0);
-  @ <p>Enable what-you-see-is-what-you-get (WYSIWYG) editing of wiki pages.
-  @ The WYSIWYG editor generates HTML instead of markup, which makes
-  @ subsequent manual editing more difficult.
-  @ (Property: "wysiwyg-wiki")</p>
+  entry_attribute("Allow Unsafe HTML In Markdown", 6,
+                  "safe-html", "safe-html", "", 0);
+  @ <p>Allow "unsafe" HTML (ex: &lt;script&gt;, &lt;form&gt;, etc) to be
+  @ generated by <a href="%R/md_rules">Markdown-formatted</a> documents.
+  @ This setting is a string where each character indicates a "type" of
+  @ document in which to allow unsafe HTML:
+  @ <ul>
+  @ <li> <b>b</b> &rarr; checked-in files, embedded documentation
+  @ <li> <b>f</b> &rarr; forum posts
+  @ <li> <b>t</b> &rarr; tickets
+  @ <li> <b>w</b> &rarr; wiki pages
+  @ </ul>
+  @ Include letters for each type of document for which unsafe HTML should
+  @ be allowed.  For example, to allow unsafe HTML only for checked-in files,
+  @ make this setting be just "<b>b</b>".  To allow unsafe HTML anywhere except
+  @ in forum posts, make this setting be "<b>btw</b>".  The default is an
+  @ empty string which means that Fossil never allows Markdown documents
+  @ to generate unsafe HTML.
+  @ (Property: "safe-html")</p>
   @ <hr />
   onoff_attribute("Use HTML as wiki markup language",
     "wiki-use-html", "wiki-use-html", 0, 0);
   @ <p>Use HTML as the wiki markup language. Wiki links will still be parsed
-  @ but all other wiki formatting will be ignored. This option is helpful
-  @ if you have chosen to use a rich HTML editor for wiki markup such as
-  @ TinyMCE.</p>
+  @ but all other wiki formatting will be ignored.</p>
   @ <p><strong>CAUTION:</strong> when
   @ enabling, <i>all</i> HTML tags and attributes are accepted in the wiki.
   @ No sanitization is done. This means that it is very possible for malicious
@@ -1097,6 +1161,8 @@ void setup_adunit(void){
   if( P("clear")!=0 && cgi_csrf_safe(1) ){
     db_multi_exec("DELETE FROM config WHERE name GLOB 'adunit*'");
     cgi_replace_parameter("adunit","");
+    cgi_replace_parameter("adright","");
+    setup_incr_cfgcnt();
   }
 
   style_header("Edit Ad Unit");
@@ -1161,7 +1227,7 @@ void setup_adunit(void){
 /*
 ** WEBPAGE: setup_logo
 **
-** Administrative page for changing the logo image.
+** Administrative page for changing the logo, background, and icon images.
 */
 void setup_logo(void){
   const char *zLogoMtime = db_get_mtime("logo-image", 0, 0);
@@ -1172,12 +1238,19 @@ void setup_logo(void){
   const char *zBgMime = db_get("background-mimetype","image/gif");
   const char *aBgImg = P("bgim");
   int szBgImg = atoi(PD("bgim:bytes","0"));
+  const char *zIconMtime = db_get_mtime("icon-image", 0, 0);
+  const char *zIconMime = db_get("icon-mimetype","image/gif");
+  const char *aIconImg = P("iconim");
+  int szIconImg = atoi(PD("iconim:bytes","0"));
   if( szLogoImg>0 ){
     zLogoMime = PD("logoim:mimetype","image/gif");
   }
   if( szBgImg>0 ){
     zBgMime = PD("bgim:mimetype","image/gif");
   }
+  if( szIconImg>0 ){
+    zIconMime = PD("iconim:mimetype","image/gif");
+  }
   login_check_credentials();
   if( !g.perm.Admin ){
     login_needed(0);
@@ -1235,6 +1308,31 @@ void setup_logo(void){
     );
     db_end_transaction(0);
     cgi_redirect("setup_logo");
+  }else if( P("seticon")!=0 && zIconMime && zIconMime[0] && szIconImg>0 ){
+    Blob img;
+    Stmt ins;
+    blob_init(&img, aIconImg, szIconImg);
+    db_prepare(&ins,
+        "REPLACE INTO config(name,value,mtime)"
+        " VALUES('icon-image',:bytes,now())"
+    );
+    db_bind_blob(&ins, ":bytes", &img);
+    db_step(&ins);
+    db_finalize(&ins);
+    db_multi_exec(
+       "REPLACE INTO config(name,value,mtime)"
+       " VALUES('icon-mimetype',%Q,now())",
+       zIconMime
+    );
+    db_end_transaction(0);
+    cgi_redirect("setup_logo");
+  }else if( P("clricon")!=0 ){
+    db_multi_exec(
+       "DELETE FROM config WHERE name IN "
+           "('icon-image','icon-mimetype')"
+    );
+    db_end_transaction(0);
+    cgi_redirect("setup_logo");
   }
   style_header("Edit Project Logo And Background");
   @ <p>The current project logo has a MIME-Type of <b>%h(zLogoMime)</b>
@@ -1285,6 +1383,30 @@ void setup_logo(void){
   @ <p>(Properties: "background-image" and "background-mimetype")
   @ <hr />
   @
+  @ <p>The current icon image has a MIME-Type of <b>%h(zIconMime)</b>
+  @ and looks like this:</p>
+  @ <blockquote><p><img src="%s(g.zTop)/favicon.ico/%z(zIconMtime)" \
+  @ alt="icon" border=1 />
+  @ </p></blockquote>
+  @
+  @ <form action="%s(g.zTop)/setup_logo" method="post"
+  @  enctype="multipart/form-data"><div>
+  @ <p>The icon image is accessible to all users at this URL:
+  @ <a href="%s(g.zBaseURL)/favicon.ico">%s(g.zBaseURL)/favicon.ico</a>.
+  @ The icon image may or may not appear on each
+  @ page depending on the web browser in use and the MIME-Types that it
+  @ supports for icon images.
+  @ To change the icon image, use the following form:</p>
+  login_insert_csrf_secret();
+  @ Icon image file:
+  @ <input type="file" name="iconim" size="60" accept="image/*" />
+  @ <p align="center">
+  @ <input type="submit" name="seticon" value="Change Icon" />
+  @ <input type="submit" name="clricon" value="Revert To Default" /></p>
+  @ </div></form>
+  @ <p>(Properties: "icon-image" and "icon-mimetype")
+  @ <hr />
+  @
   @ <p><span class="note">Note:</span>  Your browser has probably cached these
   @ images, so you may need to press the Reload button before changes will
   @ take effect. </p>
@@ -1376,12 +1498,12 @@ void sql_page(void){
   @ </form>
   if( P("schema") ){
     zQ = sqlite3_mprintf(
-            "SELECT sql FROM repository.sqlite_master"
+            "SELECT sql FROM repository.sqlite_sqlite"
             " WHERE sql IS NOT NULL ORDER BY name");
     go = 1;
   }else if( P("tablelist") ){
     zQ = sqlite3_mprintf(
-            "SELECT name FROM repository.sqlite_master WHERE type='table'"
+            "SELECT name FROM repository.sqlite_schema WHERE type='table'"
             " ORDER BY name");
     go = 1;
   }
@@ -1632,6 +1754,7 @@ void page_srchsetup(){
     onoff_attribute("Use Porter Stemmer","search-stemmer","ss",0,0);
     @ <p><input type="submit" name="fts0" value="Delete The Full-Text Index">
     @ <input type="submit" name="fts1" value="Rebuild The Full-Text Index">
+    style_submenu_element("FTS Index Debugging","%R/test-ftsdocs");
   }else{
     @ <p>The SQLite FTS4 search index is disabled.  All searching will be
     @ a full-text scan.  This usually works fine, but can be slow for
