@@ -600,6 +600,7 @@ static const char *domainOfAddr(const char *z){
 ** Options:
 **
 **      --direct              Go directly to the TO domain.  Bypass MX lookup
+**      --relayhost R         Use R as relay host directly for delivery.
 **      --port N              Use TCP port N instead of 25
 **      --trace               Show the SMTP conversation on the console
 */
@@ -609,6 +610,7 @@ void test_smtp_send(void){
   int nTo;
   const char *zToDomain;
   const char *zFromDomain;
+  const char *zRelay;
   const char **azTo;
   int smtpPort = 25;
   const char *zPort;
@@ -618,6 +620,7 @@ void test_smtp_send(void){
   if( find_option("direct",0,0)!=0 ) smtpFlags |= SMTP_DIRECT;
   zPort = find_option("port",0,1);
   if( zPort ) smtpPort = atoi(zPort);
+  zRelay = find_option("relayhost",0,1);
   verify_all_options();
   if( g.argc<5 ) usage("EMAIL FROM TO ...");
   blob_read_from_file(&body, g.argv[2], ExtFILE);
@@ -625,7 +628,12 @@ void test_smtp_send(void){
   nTo = g.argc-4;
   azTo = (const char**)g.argv+4;
   zFromDomain = domainOfAddr(zFrom);
-  zToDomain = domainOfAddr(azTo[0]);
+  if( zRelay!=0 && zRelay[0]!= 0) {
+    smtpFlags |= SMTP_DIRECT;
+    zToDomain = zRelay;
+  }else{
+    zToDomain = domainOfAddr(azTo[0]);
+  }
   p = smtp_session_new(zFromDomain, zToDomain, smtpFlags, smtpPort);
   if( p->zErr ){
     fossil_fatal("%s", p->zErr);
@@ -1230,15 +1238,18 @@ static void smtp_server_route_incoming(SmtpServer *p, int bFinish){
 /*
 ** Remove stale content from the emailblob table.
 */
-void smtp_cleanup(void){
+int smtp_cleanup(void){
+  int nAction = 0;
   if( db_table_exists("repository","emailblob") ){
     db_begin_transaction();
     db_multi_exec(
       "UPDATE emailblob SET ets=NULL WHERE enref<=0;"
       "DELETE FROM emailblob WHERE enref<=0;"
     );
+    nAction = db_changes();
     db_end_transaction(0);
   }
+  return nAction;
 }
 
 /*
@@ -1522,7 +1533,7 @@ void pop3d_command(void){
       int i;
       pop3_print(pLog, "+OK");
       for(i=0; i<sizeof(azCap)/sizeof(azCap[0]); i++){
-        pop3_print(pLog, azCap[i]);
+        pop3_print(pLog, "%s", azCap[i]);
       }
       pop3_print(pLog, ".");
       continue;
