@@ -127,6 +127,19 @@ static void sqlcmd_decompress(
   }
 }
 
+/*
+** Implementation of the "gather_artifact_stats(X)" SQL function.
+** That function merely calls the gather_artifact_stats() function
+** in stat.c to populate the ARTSTAT temporary table.
+*/
+static void sqlcmd_gather_artifact_stats(
+  sqlite3_context *context,
+  int argc,
+  sqlite3_value **argv
+){
+  gather_artifact_stats(1);
+}
+
 /*
 ** Add the content(), compress(), and decompress() SQL functions to
 ** database connection db.
@@ -138,6 +151,8 @@ int add_content_sql_commands(sqlite3 *db){
                           sqlcmd_compress, 0, 0);
   sqlite3_create_function(db, "decompress", 1, SQLITE_UTF8, 0,
                           sqlcmd_decompress, 0, 0);
+  sqlite3_create_function(db, "gather_artifact_stats", 0, SQLITE_UTF8, 0,
+                          sqlcmd_gather_artifact_stats, 0, 0);
   return SQLITE_OK;
 }
 
@@ -151,12 +166,15 @@ static int sqlcmd_autoinit(
   const char **pzErrMsg,
   const void *notUsed
 ){
+  int mTrace = SQLITE_TRACE_CLOSE;
   add_content_sql_commands(db);
   db_add_aux_functions(db);
   re_add_sql_func(db);
   search_sql_setup(db);
   foci_register(db);
   deltafunc_init(db);
+  helptext_vtab_register(db);
+  builtin_vtab_register(db);
   g.repositoryOpen = 1;
   g.db = db;
   sqlite3_db_config(db, SQLITE_DBCONFIG_MAINDBNAME, "repository");
@@ -173,6 +191,10 @@ static int sqlcmd_autoinit(
     sqlite3_exec(db, zSql, 0, 0, 0);
     sqlite3_free(zSql);
   }
+  /* Arrange to trace close operations so that static prepared statements
+  ** will get cleaned up when the shell closes the database connection */
+  if( g.fSqlTrace ) mTrace |= SQLITE_TRACE_PROFILE;
+  sqlite3_trace_v2(db, mTrace, db_sql_trace, 0);
   return SQLITE_OK;
 }
 
@@ -247,27 +269,41 @@ static void fossil_close(int bDb, int noRepository){
 **
 ** Usage: %fossil sql ?OPTIONS?
 **
-** Run the standalone sqlite3 command-line shell on DATABASE with SHELL_OPTS.
-** If DATABASE is omitted, then the repository that serves the working
-** directory is opened.  See https://www.sqlite.org/cli.html for additional
-** information.
+** Run the sqlite3 command-line shell on the Fossil repository
+** identified by the -R option, or on the current repository.
+** See https://www.sqlite.org/cli.html for additional information about
+** the sqlite3 command-line shell.
+**
+** WARNING:  Careless use of this command can corrupt a Fossil repository
+** in ways that are unrecoverable.  Be sure you know what you are doing before
+** running any SQL commands that modify the repository database.  Use the
+** --readonly option to prevent accidental damage to the repository.
 **
 ** Options:
 **
 **    --no-repository           Skip opening the repository database.
 **
+**    --readonly                Open the repository read-only.  No changes
+**                              are allowed.  This is a recommended safety
+**                              precaution to prevent repository damage.
+**
 **    -R REPOSITORY             Use REPOSITORY as the repository database
 **
-** WARNING:  Careless use of this command can corrupt a Fossil repository
-** in ways that are unrecoverable.  Be sure you know what you are doing before
-** running any SQL commands that modify the repository database.
+** All of the standard sqlite3 command-line shell options should also
+** work.
 **
-** The following extensions to the usual SQLite commands are provided:
+** The following SQL extensions are provided with this Fossil-enhanced
+** version of the sqlite3 command-line shell:
+**
+**    builtin                   A virtual table that contains one row for
+**                              each datafile that is built into the Fossil
+**                              binary.
 **
 **    checkin_mtime(X,Y)        Return the mtime for the file Y (a BLOB.RID)
 **                              found in check-in X (another BLOB.RID value).
 **
-**    compress(X)               Compress text X.
+**    compress(X)               Compress text X with the same algorithm used
+**                              to compress artifacts in the BLOB table.
 **
 **    content(X)                Return the content of artifact X. X can be an
 **                              artifact hash or hash prefix or a tag. Artifacts
@@ -292,11 +328,21 @@ static void fossil_close(int bDb, int noRepository){
 **
 **    files_of_checkin(X)       A table-valued function that returns info on
 **                              all files contained in check-in X.  Example:
-**                                SELECT * FROM files_of_checkin('trunk');
+**
+**                                  SELECT * FROM files_of_checkin('trunk');
+**
+**    helptext                  A virtual table with one row for each command,
+**                              webpage, and setting together with the built-in
+**                              help text.  
 **
 **    now()                     Return the number of seconds since 1970.
 **
-**    REGEXP                    The REGEXP operator works, unlike in
+**    obscure(T)                Obfuscate the text password T so that its
+**                              original value is not readily visible.  Fossil
+**                              uses this same algorithm when storing passwords
+**                              of remote URLs.
+**
+**    regexp                    The REGEXP operator works, unlike in
 **                              standard SQLite.
 **
 **    symbolic_name_to_rid(X)   Return the BLOB.RID corresponding to symbolic
