@@ -294,17 +294,21 @@ void stat_page(void){
 }
 
 /*
-** COMMAND: dbstat*
+** COMMAND: dbstat
 **
 ** Usage: %fossil dbstat OPTIONS
 **
-** Shows statistics and global information about the repository.
+** Shows statistics and global information about the repository and/or
+** verify the integrity of a repository.
 **
 ** Options:
 **
-**   --brief|-b           Only show essential elements
-**   --db-check           Run a PRAGMA quick_check on the repository database
-**   --omit-version-info  Omit the SQLite and Fossil version information
+**   --brief|-b           Only show essential elements.
+**   --db-check           Run "PRAGMA quick_check" on the repository database.
+**   --db-verify          Run a full verification of the repository integrity.
+**                        This involves decoding and reparsing all artifacts
+**                        and can take significant time.
+**   --omit-version-info  Omit the SQLite and Fossil version information.
 */
 void dbstat_cmd(void){
   i64 t, fsize;
@@ -319,6 +323,7 @@ void dbstat_cmd(void){
   brief = find_option("brief", "b",0)!=0;
   omitVers = find_option("omit-version-info", 0, 0)!=0;
   dbCheck = find_option("db-check",0,0)!=0;
+  if( find_option("db-verify",0,0)!=0 ) dbCheck = 2;
   db_find_and_open_repository(0,0);
 
   /* We should be done with options.. */
@@ -386,9 +391,11 @@ void dbstat_cmd(void){
                 " + 0.99");
   fossil_print("%*s%,d days or approximately %.2f years.\n",
                colWidth, "project-age:", n, n/365.2425);
-  p = db_get("project-code", 0);
-  if( p ){
-    fossil_print("%*s%s\n", colWidth, "project-id:", p);
+  if( !brief ){
+    p = db_get("project-code", 0);
+    if( p ){
+      fossil_print("%*s%s\n", colWidth, "project-id:", p);
+    }
   }
 #if 0
   /* Server-id is not useful information any more */
@@ -414,8 +421,19 @@ void dbstat_cmd(void){
                db_text(0, "PRAGMA repository.encoding"),
                db_text(0, "PRAGMA repository.journal_mode"));
   if( dbCheck ){
-    fossil_print("%*s%s\n", colWidth, "database-check:",
-                 db_text(0, "PRAGMA quick_check(1)"));
+    if( dbCheck<2 ){
+      char *zRes = db_text(0, "PRAGMA repository.quick_check(1)");
+      fossil_print("%*s%s\n", colWidth, "database-check:", zRes);
+    }else{
+      char *newArgv[3];
+      newArgv[0] = g.argv[0];
+      newArgv[1] = "test-integrity";
+      newArgv[2] = 0;
+      g.argv = newArgv;
+      g.argc = 2;
+      fossil_print("Full repository verification follows:\n");
+      test_integrity();
+    }
   }
 }
 
@@ -514,7 +532,7 @@ void repo_schema_page(void){
     style_submenu_element("Table Sizes", "repo-tabsize");
   }
   blob_init(&sql,
-    "SELECT sql FROM repository.sqlite_master WHERE sql IS NOT NULL", -1);
+    "SELECT sql FROM repository.sqlite_schema WHERE sql IS NOT NULL", -1);
   if( zArg ){
     style_submenu_element("All", "repo_schema");
     blob_appendf(&sql, " AND (tbl_name=%Q OR name=%Q)", zArg, zArg);
@@ -605,12 +623,12 @@ void repo_tabsize_page(void){
   db_multi_exec(
     "CREATE TEMP TABLE trans(name TEXT PRIMARY KEY,tabname TEXT)WITHOUT ROWID;"
     "INSERT INTO trans(name,tabname)"
-    "   SELECT name, tbl_name FROM repository.sqlite_master;"
+    "   SELECT name, tbl_name FROM repository.sqlite_schema;"
     "CREATE TEMP TABLE piechart(amt REAL, label TEXT);"
     "INSERT INTO piechart(amt,label)"
-    "  SELECT count(*), "
+    "  SELECT sum(pageno),"
     "  coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
-    "    FROM dbstat('repository')"
+    "    FROM dbstat('repository',TRUE)"
     "   GROUP BY 2 ORDER BY 2;"
   );
   nPageFree = db_int(0, "PRAGMA repository.freelist_count");
@@ -631,12 +649,12 @@ void repo_tabsize_page(void){
     db_multi_exec(
       "DELETE FROM trans;"
       "INSERT INTO trans(name,tabname)"
-      "   SELECT name, tbl_name FROM localdb.sqlite_master;"
+      "   SELECT name, tbl_name FROM localdb.sqlite_schema;"
       "DELETE FROM piechart;"
       "INSERT INTO piechart(amt,label)"
-      "  SELECT count(*), "
+      "  SELECT sum(pageno), "
       " coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
-      "    FROM dbstat('localdb')"
+      "    FROM dbstat('localdb',TRUE)"
       "   GROUP BY 2 ORDER BY 2;"
     );
     nPageFree = db_int(0, "PRAGMA localdb.freelist_count");
@@ -661,7 +679,7 @@ void repo_tabsize_page(void){
 **
 ** Only populate the artstat.atype field if the bWithTypes parameter is true.
 */
-static void gather_artifact_stats(int bWithTypes){
+void gather_artifact_stats(int bWithTypes){
   static const char zSql[] = 
     @ CREATE TEMP TABLE artstat(
     @   id INTEGER PRIMARY KEY,   -- Corresponds to BLOB.RID
@@ -672,15 +690,14 @@ static void gather_artifact_stats(int bWithTypes){
     @ );
     @ INSERT INTO artstat(id,atype,isDelta,szExp,szCmpr)
     @    SELECT blob.rid, NULL,
-    @           EXISTS(SELECT 1 FROM delta WHERE delta.rid=blob.rid),
+    @           delta.rid IS NOT NULL,
     @           size, length(content)
-    @      FROM blob
+    @      FROM blob LEFT JOIN delta ON blob.rid=delta.rid
     @     WHERE content IS NOT NULL;
   ;
   static const char zSql2[] = 
     @ UPDATE artstat SET atype='file'
-    @  WHERE id IN (SELECT fid FROM mlink)
-    @    AND atype IS NULL;
+    @  WHERE +id IN (SELECT fid FROM mlink);
     @ UPDATE artstat SET atype='manifest'
     @  WHERE id IN (SELECT objid FROM event WHERE type='ci') AND atype IS NULL;
     @ UPDATE artstat SET atype='forum'
@@ -768,10 +785,11 @@ void artifact_stats_page(void){
   ** user without check-in privileges, to prevent excessive usage by
   ** robots and random passers-by on the internet
   */
-  if( !g.perm.Write ){
-    login_needed(g.anon.Admin);
+  if( !g.perm.Write && !db_get_boolean("artifact_stats_enable",0) ){
+    login_needed(g.anon.Write);
     return;
   }
+  load_control();
 
   style_header("Artifact Statistics");
   style_submenu_element("Repository Stats", "stat");
@@ -898,8 +916,8 @@ void artifact_stats_page(void){
     @ <td data-sortkey='%08x(nTotal)' align='right'>%,d(nTotal)</td>
     @ <td data-sortkey='%08x(nFull)' align='right'>%,d(nFull)</td>
     @ <td data-sortkey='%08x(nDelta)' align='right'>%,d(nDelta)</td>
-    @ <td data-sortkey='%016x(szCmpr)' align='right'>%,lld(szCmpr)</td>
-    @ <td data-sortkey='%016x(szExp)' align='right'>%,lld(szExp)</td>
+    @ <td data-sortkey='%016llx(szCmpr)' align='right'>%,lld(szCmpr)</td>
+    @ <td data-sortkey='%016llx(szExp)' align='right'>%,lld(szExp)</td>
   }
   @ </tbody></table>
   db_finalize(&q);
