@@ -445,7 +445,7 @@ void test_tarball_cmd(void){
     Blob file;
     blob_zero(&file);
     blob_read_from_file(&file, g.argv[i], eFType);
-    tar_add_file(g.argv[i], &file, file_perm(0,0), file_mtime(0,0));
+    tar_add_file(g.argv[i], &file, file_perm(0,eFType), file_mtime(0,eFType));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -468,7 +468,7 @@ void test_tarball_cmd(void){
 ** added to as part of the tarball. It may be 0 or an empty string, in
 ** which case it is ignored. The intention is to create a tarball which
 ** politely expands into a subdir instead of filling your current dir
-** with source files. For example, pass a UUID or "ProjectName".
+** with source files. For example, pass an artifact hash or "ProjectName".
 **
 */
 void tarball_of_checkin(
