@@ -32,7 +32,8 @@
 #define TH_INIT_FORCE_TCL   ((u32)0x00000002) /* Force Tcl to be enabled? */
 #define TH_INIT_FORCE_RESET ((u32)0x00000004) /* Force TH1 commands re-added? */
 #define TH_INIT_FORCE_SETUP ((u32)0x00000008) /* Force eval of setup script? */
-#define TH_INIT_MASK        ((u32)0x0000000F) /* All possible init flags. */
+#define TH_INIT_NO_REPO     ((u32)0x00000010) /* Skip opening repository. */
+#define TH_INIT_MASK        ((u32)0x0000001F) /* All possible init flags. */
 
 /*
 ** Useful and/or "well-known" combinations of flag values.
@@ -46,9 +47,9 @@
 ** Flags set by functions in this file to keep track of integration state
 ** information.  These flags should not be used outside of this file.
 */
-#define TH_STATE_CONFIG     ((u32)0x00000010) /* We opened the config. */
-#define TH_STATE_REPOSITORY ((u32)0x00000020) /* We opened the repository. */
-#define TH_STATE_MASK       ((u32)0x00000030) /* All possible state flags. */
+#define TH_STATE_CONFIG     ((u32)0x00000020) /* We opened the config. */
+#define TH_STATE_REPOSITORY ((u32)0x00000040) /* We opened the repository. */
+#define TH_STATE_MASK       ((u32)0x00000060) /* All possible state flags. */
 
 #ifdef FOSSIL_ENABLE_TH1_HOOKS
 /*
@@ -129,6 +130,7 @@ void Th_ForceCgi(int fullHttpReply){
 void Th_InitTraceLog(){
   g.thTrace = find_option("th-trace", 0, 0)!=0;
   if( g.thTrace ){
+    g.fAnyTrace = 1;
     blob_zero(&g.thLog);
   }
 }
@@ -532,6 +534,33 @@ static int verifyCsrfCmd(
   return TH_OK;
 }
 
+/*
+** TH1 command: verifyLogin
+**
+** Returns non-zero if the specified user name and password represent a
+** valid login for the repository.
+*/
+static int verifyLoginCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  const char *zUser;
+  const char *zPass;
+  int uid;
+  if( argc!=3 ){
+    return Th_WrongNumArgs(interp, "verifyLogin userName password");
+  }
+  zUser = argv[1];
+  zPass = argv[2];
+  uid = login_search_uid(&zUser, zPass);
+  Th_SetResultInt(interp, uid!=0);
+  if( uid==0 ) sqlite3_sleep(100);
+  return TH_OK;
+}
+
 /*
 ** TH1 command: markdown STRING
 **
@@ -798,11 +827,9 @@ static int hasfeatureCmd(
     rc = 1;
   }
 #endif
-#if defined(FOSSIL_ENABLE_LEGACY_MV_RM)
   else if( 0 == fossil_strnicmp( zArg, "legacyMvRm\0", 11 ) ){
     rc = 1;
   }
-#endif
 #if defined(FOSSIL_ENABLE_EXEC_REL_PATHS)
   else if( 0 == fossil_strnicmp( zArg, "execRelPaths\0", 13 ) ){
     rc = 1;
@@ -1475,7 +1502,7 @@ static int unversionedContentCmd(
   }
   if( Th_IsRepositoryOpen() ){
     Blob content;
-    if( unversioned_content(argv[2], &content)==0 ){
+    if( unversioned_content(argv[2], &content)!=0 ){
       Th_SetResult(interp, blob_str(&content), blob_size(&content));
       blob_reset(&content);
       return TH_OK;
@@ -1746,7 +1773,16 @@ static int queryCmd(
         int szVal = sqlite3_column_bytes(pStmt, i);
         Th_SetVar(interp, zCol, szCol, zVal, szVal);
       }
+      if( g.thTrace ){
+        Th_Trace("query_eval {<pre>%#h</pre>}<br />\n", argl[2], argv[2]);
+      }
       res = Th_Eval(interp, 0, argv[2], argl[2]);
+      if( g.thTrace ){
+        int nTrRes;
+        char *zTrRes = (char*)Th_GetResult(g.interp, &nTrRes);
+        Th_Trace("[query_eval] => %h {%#h}<br />\n",
+                 Th_ReturnCodeName(res, 0), nTrRes, zTrRes);
+      }
       if( res==TH_BREAK || res==TH_CONTINUE ) res = TH_OK;
     }
     rc = sqlite3_finalize(pStmt);
@@ -2060,6 +2096,7 @@ void Th_FossilInit(u32 flags){
   int forceReset = flags & TH_INIT_FORCE_RESET;
   int forceTcl = flags & TH_INIT_FORCE_TCL;
   int forceSetup = flags & TH_INIT_FORCE_SETUP;
+  int noRepo = flags & TH_INIT_NO_REPO;
   static unsigned int aFlags[] = { 0, 1, WIKI_LINKSONLY };
   static int anonFlag = LOGIN_ANON;
   static int zeroInt = 0;
@@ -2113,6 +2150,7 @@ void Th_FossilInit(u32 flags){
     {"unversioned",   unversionedCmd,       0},
     {"utime",         utimeCmd,             0},
     {"verifyCsrf",    verifyCsrfCmd,        0},
+    {"verifyLogin",   verifyLoginCmd,       0},
     {"wiki",          wikiCmd,              (void*)&aFlags[0]},
     {0, 0, 0}
   };
@@ -2126,7 +2164,7 @@ void Th_FossilInit(u32 flags){
     ** passed a non-zero value for the needConfig parameter, make sure
     ** the necessary database connections are open prior to continuing.
     */
-    Th_OpenConfig(1);
+    Th_OpenConfig(!noRepo);
   }
   if( forceReset || forceTcl || g.interp==0 ){
     int created = 0;
@@ -2596,9 +2634,15 @@ int Th_Render(const char *z){
       z += i+5;
       for(i=0; z[i] && (z[i]!='<' || !isEndScriptTag(&z[i])); i++){}
       if( g.thTrace ){
-        Th_Trace("eval {<pre>%#h</pre>}<br />", i, z);
+        Th_Trace("render_eval {<pre>%#h</pre>}<br />\n", i, z);
       }
       rc = Th_Eval(g.interp, 0, (const char*)z, i);
+      if( g.thTrace ){
+        int nTrRes;
+        char *zTrRes = (char*)Th_GetResult(g.interp, &nTrRes);
+        Th_Trace("[render_eval] => %h {%#h}<br />\n",
+                 Th_ReturnCodeName(rc, 0), nTrRes, zTrRes);
+      }
       if( rc!=TH_OK ) break;
       z += i;
       if( z[0] ){ z += 6; }
