@@ -87,21 +87,36 @@ const char *unversioned_content_hash(int debugFlag){
 /*
 ** Initialize pContent to be the content of an unversioned file zName.
 **
-** Return 0 on success.  Return 1 if zName is not found.
+** Return 0 on failures.
+** Return 1 if the file is found by name.
+** Return 2 if the file is found by hash.
 */
 int unversioned_content(const char *zName, Blob *pContent){
   Stmt q;
-  int rc = 1;
+  int rc = 0;
   blob_init(pContent, 0, 0);
-  db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE name=%Q", zName);
+  db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE name=%Q",
+                 zName);
   if( db_step(&q)==SQLITE_ROW ){
     db_column_blob(&q, 1, pContent);
     if( db_column_int(&q, 0)==1 ){
       blob_uncompress(pContent, pContent);
     }
-    rc = 0;
+    rc = 1;
   }
   db_finalize(&q);
+  if( rc==0 && validate16(zName,-1) ){
+    db_prepare(&q, "SELECT encoding, content FROM unversioned WHERE hash=%Q",
+                   zName);
+    if( db_step(&q)==SQLITE_ROW ){
+      db_column_blob(&q, 1, pContent);
+      if( db_column_int(&q, 0)==1 ){
+        blob_uncompress(pContent, pContent);
+      }
+      rc = 2;
+    }
+    db_finalize(&q);
+  }
   return rc;
 }
 
@@ -121,7 +136,7 @@ static void unversioned_write(
     "REPLACE INTO unversioned(name,rcvid,mtime,hash,sz,encoding,content)"
     " VALUES(:name,:rcvid,:mtime,:hash,:sz,:encoding,:content)"
   );
-  sha1sum_blob(pContent, &hash);
+  hname_hash(pContent, 0, &hash);
   blob_compress(pContent, &compressed);
   db_bind_text(&ins, ":name", zUVFile);
   db_bind_int(&ins, ":rcvid", g.rcvid);
@@ -145,7 +160,7 @@ static void unversioned_write(
 
 /*
 ** Check the status of unversioned file zName.  "mtime" and "zHash" are the
-** time of last change and SHA1 hash of a copy of this file on a remote
+** time of last change and hash of a copy of this file on a remote
 ** server.  Return an integer status code as follows:
 **
 **    0:     zName does not exist in the unversioned table.
@@ -239,7 +254,10 @@ static int contains_whitespace(const char *zName){
 **    export FILE OUTPUT     Write the content of FILE into OUTPUT on disk
 **
 **    list | ls              Show all unversioned files held in the local
-**                           repository.
+**                           repository. Options:
+**
+**                              --glob PATTERN   Show only files that match
+**                              --like PATTERN   Show only files that match
 **
 **    revert ?URL?           Restore the state of all unversioned files in the
 **                           local repository to match the remote repository
@@ -252,7 +270,10 @@ static int contains_whitespace(const char *zName){
 **    remove|rm|delete FILE ...
 **                           Remove unversioned files from the local repository.
 **                           Changes are not pushed to other repositories until
-**                           the next sync.
+**                           the next sync.  Options:
+**
+**                              --glob PATTERN   Remove files that match
+**                              --like PATTERN   Remove files that match
 **
 **    sync ?URL?             Synchronize the state of all unversioned files with
 **                           the remote repository URL.  The most recent version
@@ -269,6 +290,7 @@ static int contains_whitespace(const char *zName){
 **
 **   --mtime TIMESTAMP       Use TIMESTAMP instead of "now" for the "add",
 **                           "edit", "remove", and "touch" subcommands.
+**   -R|--repository FILE    Use FILE as the repository
 */
 void unversioned_cmd(void){
   const char *zCmd;
@@ -293,8 +315,8 @@ void unversioned_cmd(void){
     int i;
 
     zAs = find_option("as",0,1);
-    if( zAs && g.argc!=4 ) usage("add DISKFILE --as UVFILE");
     verify_all_options();
+    if( zAs && g.argc!=4 ) usage("add DISKFILE --as UVFILE");
     db_begin_transaction();
     content_rcvid_init("#!fossil unversioned add");
     for(i=3; i<g.argc; i++){
@@ -323,7 +345,7 @@ void unversioned_cmd(void){
     db_begin_transaction();
     for(i=3; i<g.argc; i++){
       Blob content;
-      if( unversioned_content(g.argv[i], &content)==0 ){
+      if( unversioned_content(g.argv[i], &content)!=0 ){
         blob_write_to_file(&content, "-");
       }
       blob_reset(&content);
@@ -340,12 +362,14 @@ void unversioned_cmd(void){
     if( g.argc!=4) usage("edit UVFILE");
     zUVFile = g.argv[3];
     zEditor = fossil_text_editor();
-    if( zEditor==0 ) fossil_fatal("no text editor - set the VISUAL env variable");
+    if( zEditor==0 ){
+      fossil_fatal("no text editor - set the VISUAL env variable");
+    }
     zTFile = fossil_temp_filename();
     if( zTFile==0 ) fossil_fatal("cannot find a temporary filename");
     db_begin_transaction();
     content_rcvid_init("#!fossil unversioned edit");
-    if( unversioned_content(zUVFile, &content) ){
+    if( unversioned_content(zUVFile, &content)==0 ){
       fossil_fatal("no such uv-file: %Q", zUVFile);
     }
     if( looks_like_binary(&content) ){
@@ -355,7 +379,7 @@ void unversioned_cmd(void){
     blob_add_cr(&content);
 #endif
     blob_write_to_file(&content, zTFile);
-    zCmd = mprintf("%s \"%s\"", zEditor, zTFile);
+    zCmd = mprintf("%s %$", zEditor, zTFile);
     if( fossil_system(zCmd) ){
       fossil_fatal("editor aborted: %Q", zCmd);
     }
@@ -374,7 +398,7 @@ void unversioned_cmd(void){
     Blob content;
     verify_all_options();
     if( g.argc!=5 ) usage("export UVFILE OUTPUT");
-    if( unversioned_content(g.argv[3], &content) ){
+    if( unversioned_content(g.argv[3], &content)==0 ){
       fossil_fatal("no such uv-file: %Q", g.argv[3]);
     }
     blob_write_to_file(&content, g.argv[4]);
@@ -387,13 +411,27 @@ void unversioned_cmd(void){
     Stmt q;
     int allFlag = find_option("all","a",0)!=0;
     int longFlag = find_option("l",0,0)!=0 || (nCmd>1 && zCmd[1]=='i');
+    char *zPattern = sqlite3_mprintf("true");
+    const char *zGlob;
+    zGlob = find_option("glob",0,1);
+    if( zGlob ){
+      sqlite3_free(zPattern);
+      zPattern = sqlite3_mprintf("(name GLOB %Q)", zGlob);
+    }
+    zGlob = find_option("like",0,1);
+    if( zGlob ){
+      sqlite3_free(zPattern);
+      zPattern = sqlite3_mprintf("(name LIKE %Q)", zGlob);
+    }
     verify_all_options();
     if( !longFlag ){
       if( allFlag ){
-        db_prepare(&q, "SELECT name FROM unversioned ORDER BY name");
+        db_prepare(&q, "SELECT name FROM unversioned WHERE %s ORDER BY name",
+                   zPattern/*safe-for-%s*/);
       }else{
-        db_prepare(&q, "SELECT name FROM unversioned WHERE hash IS NOT NULL"
-                       " ORDER BY name");
+        db_prepare(&q, "SELECT name FROM unversioned"
+                       " WHERE %s AND hash IS NOT NULL"
+                       " ORDER BY name", zPattern/*safe-for-%s*/);
       }
       while( db_step(&q)==SQLITE_ROW ){
         fossil_print("%s\n", db_column_text(&q,0));
@@ -401,8 +439,8 @@ void unversioned_cmd(void){
     }else{
       db_prepare(&q,
         "SELECT hash, datetime(mtime,'unixepoch'), sz, length(content), name"
-        "   FROM unversioned"
-        "  ORDER BY name;"
+        "   FROM unversioned WHERE %s"
+        "  ORDER BY name;", zPattern/*safe-for-%s*/
       );
       while( db_step(&q)==SQLITE_ROW ){
         const char *zHash = db_column_text(&q, 0);
@@ -424,16 +462,33 @@ void unversioned_cmd(void){
       }
     }
     db_finalize(&q);
+    sqlite3_free(zPattern);
   }else if( memcmp(zCmd, "revert", nCmd)==0 ){
-    unsigned syncFlags = unversioned_sync_flags(SYNC_UNVERSIONED|SYNC_UV_REVERT);
+    unsigned syncFlags = 
+        unversioned_sync_flags(SYNC_UNVERSIONED|SYNC_UV_REVERT);
     g.argv[1] = "sync";
     g.argv[2] = "--uv-noop";
     sync_unversioned(syncFlags);
   }else if( memcmp(zCmd, "remove", nCmd)==0 || memcmp(zCmd, "rm", nCmd)==0
          || memcmp(zCmd, "delete", nCmd)==0 ){
     int i;
-    verify_all_options();
+    const char *zGlob;
     db_begin_transaction();
+    while( (zGlob = find_option("glob",0,1))!=0 ){
+      db_multi_exec(
+        "UPDATE unversioned"
+        "   SET hash=NULL, content=NULL, mtime=%lld, sz=0 WHERE name GLOB %Q",
+        mtime, zGlob
+      );
+    }
+    while( (zGlob = find_option("like",0,1))!=0 ){
+      db_multi_exec(
+        "UPDATE unversioned"
+        "   SET hash=NULL, content=NULL, mtime=%lld, sz=0 WHERE name LIKE %Q",
+        mtime, zGlob
+      );
+    }
+    verify_all_options();
     for(i=3; i<g.argc; i++){
       db_multi_exec(
         "UPDATE unversioned"
@@ -486,6 +541,7 @@ void uvlist_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  etag_check(ETAG_DATA,0);
   style_header("Unversioned Files");
   if( !db_table_exists("repository","unversioned") ){
     @ No unversioned files on this server
@@ -528,7 +584,7 @@ void uvlist_page(void){
       @   <th> Age
       @   <th> Size
       @   <th> User
-      @   <th> SHA1
+      @   <th> Hash
       if( g.perm.Admin ){
         @ <th> rcvid
       }
@@ -598,6 +654,7 @@ void uvlist_json_page(void){
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   cgi_set_content_type("text/json");
+  etag_check(ETAG_DATA,0);
   if( !db_table_exists("repository","unversioned") ){
     blob_init(&json, "[]", -1);
     cgi_set_content(&json);
