@@ -93,19 +93,24 @@ int update_to(int vid){
 ** unchanged files in addition to those file that actually do change.
 **
 ** Options:
-**   --case-sensitive <BOOL> override case-sensitive setting
-**   --debug          print debug information on stdout
-**   --latest         acceptable in place of VERSION, update to latest version
-**   --force-missing  force update if missing content after sync
-**   -n|--dry-run     If given, display instead of run actions
-**   -v|--verbose     print status information about all files
-**   -W|--width <num> Width of lines (default is to auto-detect). Must be >20
-**                    or 0 (= no limit, resulting in a single line per entry).
-**   --setmtime       Set timestamps of all files to match their SCM-side
-**                    times (the timestamp of the last checkin which modified
-**                    them).
+**   --case-sensitive BOOL  Override case-sensitive setting
+**   --debug                Print debug information on stdout
+**   --latest               Acceptable in place of VERSION, update to
+**                          latest version
+**   --force-missing        Force update if missing content after sync
+**   -n|--dry-run           If given, display instead of run actions
+**   -v|--verbose           Print status information about all files
+**   -W|--width WIDTH       Width of lines (default is to auto-detect).
+**                          Must be more than 20 or 0 (= no limit,
+**                          resulting in a single line per entry).
+**   --setmtime             Set timestamps of all files to match their
+**                          SCM-side times (the timestamp of the last
+**                          checkin which modified them).
+**  -K|--keep-merge-files   On merge conflict, retain the temporary files
+**                          used for merging, named *-baseline, *-original,
+**                          and *-merge.
 **
-** See also: revert
+** See also: [[revert]]
 */
 void update_cmd(void){
   int vid;              /* Current version */
@@ -117,6 +122,7 @@ void update_cmd(void){
   int forceMissingFlag; /* --force-missing.  Continue if missing content */
   int debugFlag;        /* --debug option */
   int setmtimeFlag;     /* --setmtime.  Set mtimes on files */
+  int keepMergeFlag;    /* True if --keep-merge-files is present */
   int nChng;            /* Number of file renames */
   int *aChng;           /* Array of file renames */
   int i;                /* Loop counter */
@@ -149,6 +155,7 @@ void update_cmd(void){
   forceMissingFlag = find_option("force-missing",0,0)!=0;
   debugFlag = find_option("debug",0,0)!=0;
   setmtimeFlag = find_option("setmtime",0,0)!=0;
+  keepMergeFlag = find_option("keep-merge-files", "K",0)!=0;
 
   /* We should be done with options.. */
   verify_all_options();
@@ -492,6 +499,7 @@ void update_cmd(void){
         nConflict++;
       }else{
         unsigned mergeFlags = dryRunFlag ? MERGE_DRYRUN : 0;
+        if(keepMergeFlag!=0) mergeFlags |= MERGE_KEEP_FILES;
         if( !dryRunFlag && !internalUpdate ) undo_save(zName);
         content_get(ridt, &t);
         content_get(ridv, &v);
@@ -683,6 +691,7 @@ Manifest *historical_manifest(
   }else{
     vid = db_lget_int("checkout", 0);
     if( !is_a_version(vid) ){
+      if( vid==0 ) return 0;
       zRevision = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", vid);
       if( zRevision ){
         fossil_fatal("checkout artifact is not a check-in: %s", zRevision);
@@ -759,24 +768,27 @@ int historical_blob(
 /*
 ** COMMAND: revert
 **
-** Usage: %fossil revert ?-r REVISION? ?FILE ...?
+** Usage: %fossil revert ?OPTIONS? ?FILE ...?
 **
 ** Revert to the current repository version of FILE, or to
-** the version associated with baseline REVISION if the -r flag
-** appears.
+** the baseline VERSION specified with -r flag.
 **
 ** If FILE was part of a rename operation, both the original file
 ** and the renamed file are reverted.
 **
+** Using a directory name for any of the FILE arguments is the same
+** as using every subdirectory and file beneath that directory.
+**
 ** Revert all files if no file name is provided.
 **
 ** If a file is reverted accidentally, it can be restored using
 ** the "fossil undo" command.
 **
 ** Options:
-**   -r REVISION    revert given FILE(s) back to given REVISION
+**   -r|--revision VERSION    Revert given FILE(s) back to given
+**                            VERSION
 **
-** See also: redo, undo, update
+** See also: [[redo]], [[undo]], [[checkout]], [[update]]
 */
 void revert_cmd(void){
   Manifest *pCoManifest;          /* Manifest of current checkout */
@@ -788,6 +800,8 @@ void revert_cmd(void){
   Blob record = BLOB_INITIALIZER; /* Contents of each reverted file */
   int i;
   Stmt q;
+  int revertAll = 0;
+  int revisionOptNotSupported = 0;
 
   undo_capture_command_line();
   zRevision = find_option("revision", "r", 1);
@@ -797,7 +811,8 @@ void revert_cmd(void){
     usage("?OPTIONS? [FILE] ...");
   }
   if( zRevision && g.argc<3 ){
-    fossil_fatal("the --revision option does not work for the entire tree");
+    fossil_fatal("directories or the entire tree can only be reverted"
+                 " back to current version");
   }
   db_must_be_within_tree();
 
@@ -815,17 +830,60 @@ void revert_cmd(void){
       zFile = mprintf("%/", g.argv[i]);
       blob_zero(&fname);
       file_tree_name(zFile, &fname, 0, 1);
-      db_multi_exec(
-        "REPLACE INTO torevert VALUES(%B);"
-        "INSERT OR IGNORE INTO torevert"
-        " SELECT pathname"
-        "   FROM vfile"
-        "  WHERE origname=%B;",
-        &fname, &fname
-      );
+      if( blob_eq(&fname, ".") ){
+        if( zRevision ){
+          revisionOptNotSupported = 1;
+          break;
+        }
+        revertAll = 1;
+        break;
+      }else if( db_exists(
+        "SELECT pathname"
+        "  FROM vfile"
+        " WHERE (substr(pathname,1,length('%q/'))='%q/'"
+        "    OR  substr(origname,1,length('%q/'))='%q/');",
+        blob_str(&fname), blob_str(&fname),
+        blob_str(&fname), blob_str(&fname)) ){
+        int vid;
+        vid = db_lget_int("checkout", 0);
+        vfile_check_signature(vid, 0);
+
+        if( zRevision ){
+          revisionOptNotSupported = 1;
+          break;
+        }
+        db_multi_exec(
+          "INSERT OR IGNORE INTO torevert"
+          " SELECT pathname"
+          "   FROM vfile"
+          "  WHERE (substr(pathname,1,length('%q/'))='%q/'"
+          "     OR  substr(origname,1,length('%q/'))='%q/')"
+          "    AND (chnged OR deleted OR rid=0 OR pathname!=origname);",
+          blob_str(&fname), blob_str(&fname),
+          blob_str(&fname), blob_str(&fname)
+        );
+      }else{
+        db_multi_exec(
+          "REPLACE INTO torevert VALUES(%B);"
+          "INSERT OR IGNORE INTO torevert"
+          " SELECT pathname"
+          "   FROM vfile"
+          "  WHERE origname=%B;",
+          &fname, &fname
+        );
+      }
       blob_reset(&fname);
     }
   }else{
+    revertAll = 1;
+  }
+
+  if( revisionOptNotSupported ){
+    fossil_fatal("directories or the entire tree can only be reverted"
+                 " back to current version");
+  }
+
+  if ( revertAll ){
     int vid;
     vid = db_lget_int("checkout", 0);
     vfile_check_signature(vid, 0);
@@ -837,6 +895,7 @@ void revert_cmd(void){
       "  WHERE chnged OR deleted OR rid=0 OR pathname!=origname;"
     );
   }
+
   db_multi_exec(
     "INSERT OR IGNORE INTO torevert"
     " SELECT origname"
@@ -853,7 +912,7 @@ void revert_cmd(void){
     char *zFull;
     zFile = db_column_text(&q, 0);
     zFull = mprintf("%/%/", g.zLocalRoot, zFile);
-    pRvFile = manifest_file_find(pRvManifest, zFile);
+    pRvFile = pRvManifest? manifest_file_find(pRvManifest, zFile) : 0;
     if( !pRvFile ){
       if( db_int(0, "SELECT rid FROM vfile WHERE pathname=%Q OR origname=%Q",
                  zFile, zFile)==0 ){
@@ -870,6 +929,8 @@ void revert_cmd(void){
         "DELETE FROM vfile WHERE pathname=%Q",
         zFile, zFile
       );
+    }else if( file_unsafe_in_tree_path(zFull) ){
+      /* Ignore this file */
     }else{
       sqlite3_int64 mtime;
       int rvChnged = 0;
