@@ -594,7 +594,7 @@ void filezip_cmd(void){
   for(i=3; i<g.argc; i++){
     blob_zero(&file);
     blob_read_from_file(&file, g.argv[i], eFType);
-    zip_add_file(&sArchive, g.argv[i], &file, file_perm(0,0));
+    zip_add_file(&sArchive, g.argv[i], &file, file_perm(0,eFType));
     blob_reset(&file);
   }
   zip_close(&sArchive);
@@ -616,7 +616,7 @@ void filezip_cmd(void){
 ** added to as part of the zip file. It may be 0 or an empty string,
 ** in which case it is ignored. The intention is to create a zip which
 ** politely expands into a subdir instead of filling your current dir
-** with source files. For example, pass a UUID or "ProjectName".
+** with source files. For example, pass a commit hash or "ProjectName".
 **
 */
 static void zip_of_checkin(
